function [BBAll BBsegm] = extract_boundbox_seg(imgSeg, BBmargin) ;
%
%   Generate bounding box candidates from the combinations of segments
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   imgSeg          : oversegmentation info
%   BBmargin        : By how much pixels each BB increases (defualt: 5)
%
% --------
% OUTPUTS:
%
%   BBAll           : bounding boxes of all possible combinations
%   BBsegm          : bounding boxes of only segments
%
% ---------------
% Gunhee Kim
%

% If a pair of BBs is overlapped too much, merge them.
OVERLAP_AREA_THRES = 0.95 ;

if nargin<1, BBmargin = 5 ; end


% number of segments
% nS = length(imgSeg.segm_id_set) ;
nS = max(imgSeg.segImg(:)) ;

% to aovid 0
% imgSeg.segm_id = [0; imgSeg.segm_id] ;
% segment image
% segmImg = imgSeg.segm_id(spImg+1) ;
[imR imC] = size(imgSeg.segImg) ;

% find the minimum bounding boxes to enclose segments
BBsegm = zeros(nS, 4) ;
for i=1:nS,
    [rr cc] = find(imgSeg.segImg==i) ;
    if isempty(rr), continue; end
    BBsegm(i,:) = [min(cc) min(rr) max(cc) max(rr)] ;
end
BBsegm(sum(BBsegm,2)==0,:) = [] ;

% enlarged by BBmargin
BBsegm = enlarge_bb(BBsegm, BBmargin, imR, imC) ;

% connectivity network
conn_net = full(imgSeg.adjMat) ;
conn_net = conn_net + conn_net' ;

% pairwise paths
nBB = size(BBsegm,1) ;
itemBB = zeros(nBB*(nBB-1)/2, 4) ;
indBB = 0 ;
for i=1:nBB-1,
    % comput all of them
    [costs,paths] = fn_dijkstra(conn_net,conn_net,i, i+1:nBB) ;
    if ~iscell(paths), tmp{1} = paths; paths = tmp; clear tmp; end
    for j=1:length(costs),
        if costs(j)<Inf,
            indBB = indBB + 1 ;
            itemBB(indBB,:) = [min(BBsegm(paths{j},1)) min(BBsegm(paths{j},2)) ...
                max(BBsegm(paths{j},3)) max(BBsegm(paths{j},4)) ] ;
        end
    end
end
itemBB = itemBB(1:indBB,:) ;

% enlarged by BBmargin
itemBB = enlarge_bb(itemBB, BBmargin, imR, imC) ;

% add segment boundging boxes
itemBB = [itemBB; BBsegm] ;

% add the image itself
itemBB = [itemBB; min(BBsegm(:,1)) min(BBsegm(:,2)) ...
        max(BBsegm(:,3)) max(BBsegm(:,4))] ;

% remove repetitive ones
BBAll = unique(itemBB, 'rows') ;

% remove overlapped ones
% itemBB = mex_remove_overlap_bb(double(itemBB), OVERLAP_AREA_THRES) ;

BBAll = single(BBAll) ;
BBsegm = single(BBsegm) ;

end


function [bb] = enlarge_bb(bb, margin, imR, imC)

% enlarged by margin
bb(:,1:2) = bb(:,1:2) - margin ;
bb(:,3:4) = bb(:,3:4) + margin ;

% let them be within image domain
bb(bb(:,1)<1, 1) = 1 ;
bb(bb(:,2)<1, 2) = 1 ;
bb(bb(:,3)>imC, 3) = imC ;
bb(bb(:,4)>imR, 4) = imR ;

end


function [max_dist] = MAXDIST(R, S)
% MAXDIST ( R, S ) is the maximum distance between rectangle R and rectangle S.
% MAXDIST ( R, S) = ∑ 1≤i ≤d xi2 where
% xi = max { | ri - s'i| , | ri' - si| }
% % http://www.cs.mcgill.ca/~cs644/Godfried/2005/Fall/fzamal/concepts.htm

x1 = max( abs(R(1)-S(3)), abs(R(3)-S(1)) ) ;
x2 = max( abs(R(2)-S(4)), abs(R(4)-S(2)) ) ;

max_dist = sqrt ( x1*x1 + x2*x2 ) ;

end

function [min_dist] = MINDIST(R, S)
% MINDIST ( R, S ) is the minimum distance between rectangle R and rectangle S.
% MINDIST ( R, S) = ∑ 1≤i ≤d xi2 where
% % http://www.cs.mcgill.ca/~cs644/Godfried/2005/Fall/fzamal/concepts.htm

if R(1) > S(3),		x1 = R(1) - S(3) ;
elseif R(3) < S(1),	x1 = S(1) - R(3) ;
else,				x1 = 0 ;
end

if R(2) > S(4),		x2 = R(2) - S(4) ;
elseif R(4) < S(2),	x2 = S(2) - R(4) ;
else,				x2 = 0 ;
end

min_dist = sqrt ( x1*x1 + x2*x2 ) ;

end

% compute the enlargement, R: query, S: tree_conn
function [enlarged new_rect] = enlargement(R, S)

new_rect = [min(R(1), S(1)) min(R(2), S(2)) max(R(3), S(3)), max(R(4), S(4))] ;

enlarged = (new_rect(3)-new_rect(1))*(new_rect(4)-new_rect(2))  - (S(3)-S(1))*(S(4)-S(2)) ;

end

% get merged area
function [area new_rect] = merged_area(R, S)

new_rect = [min(R(1), S(1)) min(R(2), S(2)) max(R(3), S(3)), max(R(4), S(4))] ;

area = (new_rect(3)-new_rect(1))*(new_rect(4)-new_rect(2)) ;

end

