function [descSP] = feat_sp_clrtxt(img, spImg, clrMode, textMode)
%
%   Extract color/texture features of each segment
%
% -------
% INPUTS:
%
%   img             : The input image
%   spImg           : Superpixel image
%   clrMode         : color descriptor mode
%                     0: No, 1: L*ab, 2: RGB, 3: HSV
%   textMode        : texture descriptor mode
%                     0: No, 1: Yes.
%
% --------
% OUTPUTS:
%
%   descSP          : features of superpixels
%       .RGB        : RGB color feature
%       .HSV        : HSV color feature
%       .LAB        : L*ab color feature
%       .TEX        : texture feature
%
% ---------------
% Gunhee Kim
%

% As default, L*ab color and no texture descriptor.
if nargin<3,
    clrMode = [1 2] ;
end
if nargin<4,
    textMode = 0 ;
end


% image size
[isz_r isz_c isz_ch] = size(img) ;

% output
descSP.RGB = [] ;
descSP.HSV = [] ;
descSP.LAB = [] ;
descSP.TEX = [] ;

% color descriptor type
img_LAB = [] ;
img_RGB = [] ;
img_HSV = [] ;
for i=1:length(clrMode),
switch clrMode(i),
    case 1      % L*ab
        img_LAB = fn_RGB2Lab(img) ;
        img_LAB = reshape(double(img_LAB), [isz_r*isz_c isz_ch]) ;
    case 2      % RGB
        img_RGB = reshape(double(img), [isz_r*isz_c isz_ch]) ;
    case 3      % HSV
        img_HSV = fn_rgb2hsv_fast(img,'single') ;
        img_HSV = reshape(double(img_HSV), [isz_r*isz_c isz_ch]) ;
end
end


% texture descriptor type
switch textMode,
    case 1
        if size(img, 3)==3
            img_f = rgb2gray(img);
        end
        img_texton = MRS4fast(imfilter(img_f, fspecial('gaussian', 3, 1))) ;
    otherwise
end

% segment type
TspImg = spImg(:) ;
num_SP = max(TspImg) ;



% compute the mean of color descriptors of each segment
if ~isempty(img_LAB), descSP.LAB = single(mex_get_desc_mean_segm(TspImg, img_LAB, num_SP)) ; end
if ~isempty(img_RGB), descSP.RGB = single(mex_get_desc_mean_segm(TspImg, img_RGB, num_SP)) ; end
if ~isempty(img_HSV), descSP.HSV = single(mex_get_desc_mean_segm(TspImg, img_HSV, num_SP)) ; end
%{
descSPClr = zeros(num_SP, isz_ch) ;
for i=1:num_SP,
    tmp_ind = find(TspImg==i) ;
    descSPClr(i,:) = mean(img_clr(tmp_ind,:),1) ;
end
% The above routine is implemented in mex-c.
descSPClr = mex_get_desc_mean_segm(TspImg, img_clr, num_SP) ;
%}


% compute the mean of texture descriptors of each segment
if textMode==1,
    %{
    descSPTex = zeros(num_SP, size(img_texton,2)) ;
    for i=1:num_SP,
        tmp_ind = find(TspImg==i) ;
        descSPTex(i,:) = mean(img_clr(tmp_ind,:),1) ;
    end
    %}

    % The above routine is implemented in mex-c.
    descSP.TEX = single(mex_get_desc_mean_segm(TspImg, img_texton, num_SP)) ;
end

end
