function [setCandFg valCandFg] = get_GMM_candidates_beam_search( ...
                tree, simSeg2Comp, massSeg, search_dir, BEAM_WIDTH) ;
%
%   get foreground candidates for GMM using beam search
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   tree            : input graph
%   simSeg2Comp     : values of each segment
%   massSeg         : weights of each segment
%   search_dir      : search direction (+1: 0 -> add by one,
%                     -1: full -> remove by one)
%   BEAM_WIDTH      : beam width
%
% --------
% OUTPUTS:
%
%   setCandFg       : foreground candidates ([num_set x len_tree] uint8)
%   valCandFg       : their values ([1 x num set])
%
% ---------------
% Gunhee Kim
%

if nargin<5,
    BEAM_WIDTH = 10 ;
end
if nargin<4,
    search_dir = 1 ; % default: forward
end

% incentive for large FG candidates
PCT_INCENT_LARGE = 0.02 ;


% % --------------------------------------------------------------%
%       1. Get connectivity tree
% % --------------------------------------------------------------%

% normalize simSeg2Comp by sizes of segments
simSeg2Comp = simSeg2Comp ./ massSeg ;
simSeg2Comp(isnan(simSeg2Comp)) = 0 ;

% remove diagonal elements (lower diagonal)
tree = tril(tree,-1) ;

% make the connectivity graph
tree(tree~=0) = 1 ;
% make it symmetric
tree = tree + tree' ;
tree = full(tree) ;

len_tree = length(tree) ;

% % --------------------------------------------------------------%
%       2. Initialize Beam search
% % --------------------------------------------------------------%


% (1) values for individuals of size of 1
open_set = [1:len_tree]' ;
% get gmm value
valCandFgInd = get_GMMscore_mean(simSeg2Comp, massSeg, open_set)' ;

% (2) value for full size.
open_set = 1:len_tree ;
% get gmm value
valCandFgAll = get_GMMscore_mean(simSeg2Comp, massSeg, open_set) ;

% first candidates are every segment
if search_dir>0,
    open_set = [1:len_tree]' ;
    setCandFg = uint8(eye(len_tree)) ;
    valCandFg = valCandFgInd ;

% first candidate is the input graph itself.
else
    open_set = 1:len_tree ;
    setCandFg = ones(1,len_tree,'uint8') ;
    valCandFg = valCandFgAll ;
end


% % --------------------------------------------------------------%
%       3. Iterations
% % --------------------------------------------------------------%

while 1 ,

    % if open set is empty, break it.
    if size(open_set,2)==0, break ; end

    if search_dir>0,
        if size(open_set,2) == len_tree-1, break ; end
    % if the size of the open set is 2, break it.
    else
        if size(open_set,2) == 2, break ; end
    end

% % --------------------------------------------------------------%
%       3.1 Find next sets by adding/removing one edge
% % --------------------------------------------------------------%

    % for each of open_set
    next_set = [] ; id_next = 0 ;
    for i=1:size(open_set,1),

        if search_dir>0,
            % find the nodes that are connected by me
            s_ind = find(sum(tree(open_set(i,:),:),1)>0) ;
            s_ind = setdiff(s_ind, open_set(i,:)) ;
            for j=1:length(s_ind),
                next_set = [next_set; [open_set(i,:) s_ind(j)]] ;
            end
        else
            % generate subgraphs by removing one edge
            [s_ind] = find(sum(tree(open_set(i,:), open_set(i,:)),1)==1) ;
            for j=1:length(s_ind),
                next_set = [next_set; [open_set(i,1:s_ind(j)-1) ...
                    open_set(i,s_ind(j)+1:end)]];
            end
        end
        %{
        num_s_ind = length(s_ind) ;
        tmp_set = repmat(open_set(i,:),[num_s_ind 1]) ;
        tmp_set(sub2ind([num_s_ind, len_tree], 1:num_s_ind, s_ind)) = 0 ;
        next_set = [next_set; tmp_set] ;
        %}
    end

    % ignore repetitive ones
    next_set = sort(next_set,2) ;
    next_set = unique(next_set,'rows') ;


% % --------------------------------------------------------------%
%       3.2 Compute values
% % --------------------------------------------------------------%


    % get gmm value
    val = get_GMMscore_mean(simSeg2Comp, massSeg, next_set)' ;

    len_val = length(val) ;

    % if the element is more than BEAM_WIDTH
    if len_val > BEAM_WIDTH,
        % sort by values
        [sort_val sort_ind] = sort(val, 'descend') ;

        % consider only top-BEAM_WIDTH ones
        next_set = next_set(sort_ind(1:BEAM_WIDTH), :) ;

        val = sort_val(1:BEAM_WIDTH) ;
        len_val = BEAM_WIDTH ;
    end

    % update open set
    open_set = next_set ;

    % back them up
    tmp_setCandFg = zeros(len_val, len_tree, 'uint8') ;
    r_ind = repmat([1:len_val]', [1 size(next_set,2)]) ;
    tmp_setCandFg(sub2ind([len_val, len_tree], r_ind(:), next_set(:))) = 1 ;

    setCandFg = [setCandFg; tmp_setCandFg] ;
    valCandFg = [valCandFg val] ;

end

% add the elements with full size
if search_dir>0,
    setCandFg = [setCandFg; ones(1,len_tree,'uint8')] ;
    valCandFg = [valCandFg valCandFgAll] ;
% add the elements with size of 1
else
    setCandFg = [setCandFg; uint8(eye(len_tree))] ;
    valCandFg = [valCandFg valCandFgInd] ;
end

end


% get mean score
function outval = get_GMMscore_mean(val, wgt, indL)

[nData nDim] = size(indL) ;

if nData==1,
    outval = sum(val(indL) .* wgt(indL)) ./ sum(wgt(indL)) ;
else
    outval = sum(val(indL) .* wgt(indL), 2) ./ sum(wgt(indL), 2) ;
end

end
