function [cand_mst cand_graph minmaxSPM] = get_SPMGMM_MST_FG (segAdjmat, ...
            SimSeg2FG, massSeg, FG_SPM, sptHist, OPT, flimg, fsegm)
%
%   get candidate graph and candidate tree (mst) for GMM and SPM foreground models
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   segAdjmat       : adjacenty graph of segments
%   SimSeg2FG       : similarity between segments and foregrounds
%   massSeg         : weights (number of pixels) of each segment
%   FG_SPM          : SPM FG model (linear svm)
%   BBsegm          : bounding box coordinates of each segment
%   segm_bb         : coordinates of all possible bounding box combinations
%   sptHist         : spatial SIFT histograms of segments
%   OPT             : parameters
%   flimg           : the original image
%   fsegm           : segmentation info
%
% --------
% OUTPUTS:
%
%   cand_mst        : candidate tree (MST) (Lower Triangular Matrix)
%   cand_graph      : candidate graph (Lower Triangular Matrix)
%   minmaxSPM       : min-max SPM values for normalization
%
% ---------------
% Gunhee Kim
%

% number of segments
nSeg = length(SimSeg2FG) ;

% b draw
do_draw = 0 ;

% % --------------------------------------------------------------%
%       1.1 Construct weighted connectivity graph (GMM)
% % --------------------------------------------------------------%

% first find edges of segment adjacency graph.
[rr cc] = find(segAdjmat ~= 0) ;
[rc] = find(segAdjmat ~= 0) ;

%{
% (1) Method 1: just sum of log-likelihood
% (a) diagonal elements ( (i,i): negative log-likelihood i-th segment )
cand_graph = diag(SimSeg2FG) ;
% (b) off-diagonal elements
% Get connectivity graph
cand_graph(rc) = SimSeg2FG(rr) + SimSeg2FG(cc) ;
%}


% (2) Method 2: average of log-likelihood ;
% (a) diagonal elements
% (i,i): negative log-likelihood i-th segment)
cand_graph = diag(SimSeg2FG ./ massSeg) ;
% (b): off-diagonal elements
% (i,j): mean of negative log-likelihood of i-th and j-th segments
cand_graph(rc) = (SimSeg2FG(rr) + SimSeg2FG(cc)) ./ (massSeg(rr) + massSeg(cc)) ;
cand_graph(isnan(cand_graph)) = 0 ;


% % --------------------------------------------------------------%
%       1.2 Construct weighted connectivity graph (SPM)
% % --------------------------------------------------------------%

% diagonal
listPair = repmat(1:nSeg, [2 1])' ;
% off-diagonal
listPair = [listPair; [rr cc]] ;

% get svm values
prob_est = fn_runsvm_segm(FG_SPM, sptHist, listPair, OPT.SVM_TYPE) ;

% (a) diagonal elements
cand_graph2 = diag(prob_est(1:nSeg)) ;
% (b): off-diagonal elements
cand_graph2(rc) = prob_est(nSeg+1:end) ;


% merge outputs from two FGs
% At this point, the correct normalization between GMM and SPM is
% not important because MST is obained for each FG separately.
cand_graph = OPT.WEIGHT_FG_GMM*norm_cand_graph_0to1(cand_graph) + ...
    (1-OPT.WEIGHT_FG_GMM)*norm_cand_graph_0to1(cand_graph2) ;

% % --------------------------------------------------------------%
%       2. Run Kruskal algorithm
% % --------------------------------------------------------------%

% Build maximum spanning tree
cand_mst = fn_kruskal(-cand_graph) ;

% get the correct values
cand_mst(cand_mst~=0) = cand_graph(cand_mst~=0) ;

% make the graph sparse
cand_graph = sparse(cand_graph) ;

minmaxSPM = [min(prob_est) max(prob_est)] ;

% (DEBUG) display
if do_draw == 0, return ; end

end


% normalized cand graph to [0,1]
function cand_graph = norm_cand_graph_0to1 (cand_graph)

minVal = min(cand_graph(:)) ; maxVal = max(cand_graph(:)) ;
zeroInd = find(cand_graph==0) ;
cand_graph = (cand_graph - minVal) / (maxVal - minVal) ;
cand_graph(zeroInd) = 0 ;

end
