function [GMMdescFG SPMdescFG] = get_descFG_from_gt (Lflgt, Lfsegm, ...
        Lffeat, OPT, IND_CLASS)
%
%  Get the descriptors of foreground from groundtruth (for PASCAL dataset)
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   Lflgt       : paths of gt images
%   Lfsegm      : paths of oversegmentation info of gt images.
%   Lffeat      : paths of features of gt images.
%   OPT         : parameters
%   IND_CLASS   : class id of foregrounds
%
% --------
% OUTPUTS:
%
%   GMMdescFG   : Descriptors of foreground pixels for GMM (struct:[1 x ForeK+1])
%                 K+1 foreground == background
%       .desc   : The RGB values
%       .cnt    : the weights (pixels) of each RGB value
%       .imgID  : image ID
%
%   SPMdescFG   : Descriptors of foreground pixels for GMM (struct:[1 x ForeK+1])
%                 K+1 foreground == background
%       .desc   : SIFT feature values ([Dim x # of features])
%       .imgID  : image ID
%
% ---------------
% Gunhee Kim
%


% maximum area overlap to be chosen as background
BG_MAX_AREA_OVERLAP = 0.5 ;

if nargin<3, IND_CLASS = [] ;   end

% number of gt images
num_gt_img = length(Lflgt) ;

% The gt images may contain multiple foregrounds.
% If IND_CLASS is empty, we find the top-'ForeK' biggest FGs
% as ground truth FGs.
ForeK = OPT.ForeK ;

if isempty(IND_CLASS),
    [IND_CLASS, ForeK] = get_ind_class(Lflgt, ForeK) ;
end


% initialize
GMMdescFG = [] ;
SPMdescFG = [] ;
for i=1:ForeK+1,
    GMMdescFG(i).desc = [] ;
    GMMdescFG(i).cnt = [] ;
    GMMdescFG(i).imgID = [] ;

    SPMdescFG(i).desc = [] ;
    SPMdescFG(i).imgID = [] ;
end

% read segment features
% foreground superpixel histogram
fg_sp_hist = cell(1,num_gt_img) ;

for i=1:num_gt_img,

    % load segment
    load (Lfsegm{i}, 'imgSP', 'imgSeg') ;

    % load gt images
    gtimg = fn_get_gtImg (Lflgt{i}) ;
    gtimg = fn_eql_imsize(gtimg, imgSeg.segImg, 1) ;

    % GMM: get histogram of SPs for foregrounds
    % fg_sp_hist is a num_gt_img sized cell, each of which is sized
    % as [# of SPs x # of FGs+1] (The last column is for background).
    % The values indicate how many pixels of each SP belong to a FG.
    fg_sp_hist{i} = get_GMM_FG_SP_hist(imgSP, gtimg, ForeK, IND_CLASS) ;
    % backup imaage ID
    for j=1:size(fg_sp_hist{i},2),
        if sum(fg_sp_hist{i}(:,j))>0,
            GMMdescFG(j).imgID = [GMMdescFG(j).imgID i];
        end
        if OPT.FG_MOD(2) > 0, SPMdescFG(j).imgID = GMMdescFG(j).imgID ; end
    end
end
clear gtimg ;

% GMM: get feature data for foregrounds
if OPT.FG_MOD(1) > 0,
    [GMMdescFG] = get_GMM_FG_desc(GMMdescFG, Lffeat, fg_sp_hist, ForeK) ;
end

% SPM: get feature data for foregrounds
if OPT.FG_MOD(2) > 0,
    [SPMdescFG] = get_SPM_FG_desc(SPMdescFG, Lffeat, fg_sp_hist, ForeK, OPT) ;
end

end


% get IND_CLASS. (find the top-ForeK largest labeled regions)
function [IND_CLASS, ForeK] = get_ind_class(Lflgt, ForeK)

    num_gt_img = length(Lflgt) ;

    class_label = [] ;
    cnt_class_label = [] ;
    for i=1:num_gt_img,

        % load gt imag
        [gtimg] = fn_get_gtImg (Lflgt{i}) ;

        % unique labels of this image
        uniq_label = unique(gtimg(:)) ;

        % pixel counts of the labels
        count_this = histc(gtimg(:), uniq_label) ;

        % ignore 0 and 255
        del_ind = [find(uniq_label==0) ; find(uniq_label==255)] ;
        uniq_label(del_ind) = [] ;
        count_this(del_ind) = [] ;

        % save the unique labels and their counts.
        class_label = [class_label; uniq_label] ;
        cnt_class_label = [cnt_class_label; count_this] ;
    end

    % final unique label
    uniq_label = unique(class_label) ;
    % final count of pixels per label
    fin_cnt_label = sparse(ones(length(class_label),1), double(class_label), ...
        cnt_class_label, 1, double(max(class_label))) ;

    % sort class in a descending order of sizes
    [sval sind] = sort(fin_cnt_label, 'descend') ;
    ForeK = min(ForeK, nnz(sval)) ;

    IND_CLASS = sort(sind(1:ForeK)) ;
end


% GMM: get histogram of SPs for foregrounds
% fg_sp_hist ([# of SPs x ForeK+1]) : histogram of SPs per FG
function [fg_sp_hist] = get_GMM_FG_SP_hist(SP, gtimg, ForeK, IND_CLASS)

    % fg_sp_hist{i} : histogram of the SP ids of foregrounds
    % [# of SP x # of FGs]
    fg_sp_hist = [] ;

    % for each K Foreground
    for j=1:ForeK+1,
        % foreground
        if j<=ForeK,
            % count the SP ids of foreground pixels.
            tmp_fg_sp_id = SP.SPImg(gtimg==IND_CLASS(j)) ;
            % set 255 to visited pixels
            gtimg(gtimg==IND_CLASS(j)) = 255 ;
        % background (the others except IND_CLASS become background)
        else,
            tmp_fg_sp_id = SP.SPImg(gtimg~=255) ;
        end
        tmp_cnt_hist = histc(tmp_fg_sp_id,[1:length(SP.SP2Seg)]) ;
        fg_sp_hist = [fg_sp_hist tmp_cnt_hist] ;
    end
    % fg_sp_hist = uint32(fg_sp_hist) ;
end


% GMM: get feature data for foregrounds
function [descFG] = get_GMM_FG_desc(descFG, Lffeat, fg_sp_hist, ForeK)

    num_img = length(Lffeat) ;
    % [features of foreground superpixels weights]
    %for i=1:ForeK+1, descFG{i} = [] ; end

    for i=1:num_img ,
        % load descriptor file
        load(Lffeat{i}, 'descSP') ;

        if isempty(descFG(1).desc),
            tmp_nSP = round(size(descSP.RGB,1)*1.1) ;
            % initialization
            for j=1:ForeK+1,
                descFG(j).desc = zeros(num_img*tmp_nSP, 3, 'uint8') ;
                descFG(j).cnt = zeros(num_img*tmp_nSP, 1, 'uint32') ;
            end
            pntDescFG = zeros(1,ForeK+1) ;
        end

        for j=1:ForeK+1,
            % get the foreground SP
            tmp_sp_id = find(fg_sp_hist{i}(:,j)) ;
            % their counts
            tmp_sp_cnt = fg_sp_hist{i}(tmp_sp_id,j) ;
            len_tmp_cnt = length(tmp_sp_cnt) ;
            % make the input data for GMM
            descFG(j).desc(pntDescFG(j)+1:pntDescFG(j)+len_tmp_cnt,:) = ...
                descSP.RGB(tmp_sp_id,:) ;
            descFG(j).cnt(pntDescFG(j)+1:pntDescFG(j)+len_tmp_cnt) = ...
                tmp_sp_cnt ;
            pntDescFG(j) = pntDescFG(j) + len_tmp_cnt ;
        end
    end

    for i=1:ForeK+1,
        descFG(i).desc = descFG(i).desc(1:pntDescFG(i),:) ;
        descFG(i).cnt = descFG(i).cnt(1:pntDescFG(i),:) ;
    end
end


% SPM: get feature data for foregrounds
function [descFG] = get_SPM_FG_desc(descFG, Lffeat, fg_sp_hist, ForeK, OPT)

    num_img = length(Lffeat) ;

    BG_RAND_AREA_THRES = 0.2 ;

    % [features of foreground superpixels weights]
    for i=1:num_img ,
        % load descriptor file
        if OPT.BSIFTCOLOR>0,
            load(Lffeat{i}, 'sptHistHSV') ;  % 'sptHistHSV' ;
            sptHist = sptHistHSV ; clear sptHistHSV ;
        else, load(Lffeat{i}, 'sptHist') ; end

        if isempty(descFG(1).desc),
            dim_feat = size(sptHist,1) ;
            % initialization
            for j=1:ForeK+1,
                descFG(j).desc = zeros(dim_feat, num_img*(ForeK+1), 'single') ;
            end
            pntDescFG = zeros(1,ForeK+1) ;
        end

        for j=1:ForeK+1,
            % get the foreground SP
            tmp_sp_id = find(fg_sp_hist{i}(:,j)) ;
            if isempty(tmp_sp_id), continue; end
            % their counts
            tmp_sp_cnt = fg_sp_hist{i}(tmp_sp_id,j) ;

            % weight sum of each SP hist
            tmp_desc = sptHist(:,tmp_sp_id) .* ...
                repmat(double(tmp_sp_cnt)', [dim_feat 1]) ;

            % l1 normalization
            tmp_desc = sum(tmp_desc,2) ;
            tmp_desc = tmp_desc / sum(tmp_desc) ;

            pntDescFG(j) = pntDescFG(j) + 1 ;
            descFG(j).desc(:,pntDescFG(j)) = full(tmp_desc) ;
        end

        % populate the background by random selection
        bg_sp_id = find(fg_sp_hist{i}(:,end)) ;
        len_bg_sp_id = length(bg_sp_id) ;

        % random sizes (too small and too large ones are ignored)
        rand_size = randperm(len_bg_sp_id) ;
        rand_size(rand_size>len_bg_sp_id*(1-BG_RAND_AREA_THRES)) = [] ;
        rand_size(rand_size<len_bg_sp_id*BG_RAND_AREA_THRES) = [] ;

        for j=1:OPT.BG_BB_NUM_PIM-1,
            try,
            % random SP indices for BG
            rand_sp_id = randperm(len_bg_sp_id) ;
            % rand_sp_id = rand_sp_id(1:min(rand_size(j),len_bg_sp_id)) ;
            rand_sp_id = rand_sp_id(1:rand_size(j)) ;
            tmp_sp_cnt = fg_sp_hist{i}(bg_sp_id(rand_sp_id),end) ;

            % weight sum of each SP hist
            tmp_desc = sptHist(:,rand_sp_id) .* ...
                repmat(double(tmp_sp_cnt)', [dim_feat 1]) ;

            % l1 normalization
            tmp_desc = sum(tmp_desc,2) ;
            tmp_desc = tmp_desc / sum(tmp_desc) ;

            % save the bg descriptor
            pntDescFG(ForeK+1) = pntDescFG(ForeK+1) + 1 ;
            descFG(ForeK+1).desc(:,pntDescFG(ForeK+1)) = full(tmp_desc) ;
            catch,
                disp('get_descFG_from_gt.m') ;
            end
        end
    end

    for i=1:ForeK+1,
        descFG(i).desc = descFG(i).desc(:,1:pntDescFG(i)) ;
    end

end

