function [GMMdescFG SPMdescFG] = get_descFG_unsup (Lflgt, ...
        Lfsegm, Lffeat, OPT, IND_CLASS)
%
%  Get the descriptors of foreground from groundtruth
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   Lflgt       : paths of gt images
%   Lfsegm      : paths of oversegmentation info of gt images.
%   Lffeat      : paths of features of gt images.
%   ForeK       : number of foregrounds
%   IND_CLASS   : class id of foregrounds
%
% --------
% OUTPUTS:
%
%   GMMdescFG   : Descriptors of foreground pixels for GMM (struct:[1 x ForeK+1])
%                 K+1 foreground == background
%       .desc   : The RGB values
%       .cnt    : the weights of each RGB values
%
%   GMMdescFG   : Descriptors of foreground pixels for GMM (struct:[1 x ForeK+1])
%                 K+1 foreground == background
%       .desc   : SIFT feature values ([Dim x # of features])
%       .imgID  : the original image ID
%       .bbID   : the BB ID
%
% ---------------
% Gunhee Kim
%


% % --------------------------------------------------------------%
%   Randomly sample images and load their SIFT descriptors
% % --------------------------------------------------------------%

% number of images
nD = length(Lfsegm) ;

% randomly selet images and apply diversity ranking
randInd = randperm(nD) ;
selImgInd = randInd(1:min(OPT.UNSUP_NIMG, nD)) ;

LSptHist = cell(1, length(selImgInd)) ;
indSptHist = [] ;
for i=1:length(selImgInd),
    % load SIFT feature
    load(Lffeat{selImgInd(i)}, 'segm_bb', 'sptHistHSV') ; % 'BBsegm', 'descSP',

    [dimFeat inumBB] = size(sptHistHSV) ;

    % ---------------------------------
    % randomly choose BBs
    % This part may need to be changed according to characteristics
    % of datasets

    % (1) randomly choose the BBs from an image
    randInd = randperm(inumBB) ;
    randInd = randInd(1:min(OPT.UNSUP_NBBPIMG, inumBB)) ;
    randInd = sort(randInd) ;
    %{
    % (2) Choose the image themselves as candidates (based on
    % NIPS09 idea)
    [~, randInd] = max((segm_bb(:,3)-segm_bb(:,1)).*(segm_bb(:,4)-segm_bb(:,2))) ;
    %}
    % ---------------------------------

    % L1 normalization
    LSptHist{i} = sptHistHSV(:,randInd) ;
    LSptHist{i} = LSptHist{i} ./ repmat(sum(LSptHist{i},1), [dimFeat  1]) ;
    indSptHist = [indSptHist; [repmat(i, [length(randInd) 1]) randInd']] ;
end
clear sptHistHSV ;
LSptHist = horzcat(LSptHist{:}) ;

% % --------------------------------------------------------------%
%   Apply diversity ranking on the similarity graph
% % --------------------------------------------------------------%

% similarity matrix using histogram intersection
Kern = vl_alldist2(LSptHist, 'kl1') ;
nKern = size(Kern,1) ;

for i=1:length(selImgInd),
    ifInd = find(indSptHist(:,1)==i) ;
    Kern(ifInd,ifInd) = eye(length(ifInd)) ;
end

% make K-nn graph
Kern = fn_get_knn_matrix(Kern, round(5*log(nKern))) ;

% Select a method to solve linear eq Ax=b.
% 0: incremental inverse.
% 1: LU decomposition.
% 2: naive backslash.
% 3: power iteration
LIN_MODE = 3 ;
% The contant for ground conductance
constZ = 0.1 ;
% conductance to dummy ground
Z =  constZ*mean(Kern(Kern>0)) ;
K = OPT.ForeK*2 ;
% diversity ranking
[rank_set rank_set_val] = run_div_rank_lazy_greedy(Kern, K, ...
        LIN_MODE, Z*ones(nKern,1)) ;

% run clustering from rank set.
% [cluster_id] = run_cluster_from_rankset(Kern, rank_set(1:K), 1:K) ;

% % --------------------------------------------------------------%
%   Generate SPMdescFG
% % --------------------------------------------------------------%

% number of positive samples
numPosSample = round(OPT.GT_RATIO*nD) ;

% Each ranked item by diversity ranking becomes a FG center.
% Then, find numPosSample-NN of the FG center, which will be
% positive bounding boxes.
%   .desc : SIFT descriptors
%   .imgID : the original image ID
%   .bbID : BB ID
SPMdescFG = [] ;
indForeAll = [] ;
for i=1:OPT.ForeK,
    [~, sortInd] = sort(Kern(rank_set(i),:) , 'descend') ;
    posInd = sortInd(1:numPosSample) ;

    SPMdescFG(i).desc = LSptHist(:,posInd) ;
    SPMdescFG(i).imgID = single(selImgInd(indSptHist(posInd,1))) ;
    SPMdescFG(i).bbID = single(indSptHist(posInd,2)') ;
    indForeAll = [indForeAll posInd] ;
end


% (METHOD 1)
% For background, we randomly choose the images that does not
% contain foregrounds, and then randomly choose BBs in each image.
% find out non-selected images
nonSelImgId = 1:nD ;
nonSelImgId(horzcat(SPMdescFG(:).imgID)) = [] ;
nNonSelImgId = length(nonSelImgId) ;

BG_IMG_RATIO = 5 ;
% randomly choose BG_IMG_RATIO times more images
randInd = randperm(nNonSelImgId) ;
randImgInd = nonSelImgId(randInd(1:min(nNonSelImgId,numPosSample*BG_IMG_RATIO))) ;

LSptHist = cell(1, length(selImgInd)) ;
indSptHist = [] ;
for i=1:length(randImgInd),
    % load SIFT feature
    load(Lffeat{randImgInd(i)}, 'segm_bb', 'sptHistHSV') ; % 'BBsegm', 'descSP',

    [dimFeat inumBB] = size(sptHistHSV) ;

    % randomly choose the BBs from an image
    randInd = randperm(inumBB) ;
    randInd = randInd(1:min(OPT.UNSUP_NBBPIMG, inumBB)) ;
    randInd = sort(randInd) ;

    % L1 normalization
    LSptHist{i} = sptHistHSV(:,randInd) ;
    LSptHist{i} = LSptHist{i} ./ repmat(sum(LSptHist{i},1), [dimFeat  1]) ;
    indSptHist = [indSptHist; [repmat(i, [length(randInd) 1]) randInd']] ;
end
clear sptHistHSV ;
LSptHist = horzcat(LSptHist{:}) ;

SPMdescFG(OPT.ForeK+1).desc = LSptHist ;
SPMdescFG(OPT.ForeK+1).imgID = single(randImgInd(indSptHist(:,1))) ;
SPMdescFG(OPT.ForeK+1).bbID = single(indSptHist(:,2)') ;

randInd = randperm(size(SPMdescFG(OPT.ForeK+1).desc,2)) ;
randInd = randInd(1:min(OPT.UNSUP_NBBPIMG*numPosSample,length(randInd))) ;

SPMdescFG(OPT.ForeK+1).desc = SPMdescFG(OPT.ForeK+1).desc(:,randInd) ;
SPMdescFG(OPT.ForeK+1).imgID = SPMdescFG(OPT.ForeK+1).imgID(randInd) ;
SPMdescFG(OPT.ForeK+1).bbID = SPMdescFG(OPT.ForeK+1).bbID(randInd) ;

%{
% (METHOD 2)
% For background, we randomly choose numPosSample*OPT.BG_BB_NUM_PIM
% of bounding boxes out of unchosen ones by FGs.
indBackAll = 1:size(LSptHist,2) ;
indBackAll(indForeAll) = [] ;

randInd = randperm(length(indBackAll)) ;
indBackAll = indBackAll(randInd(1:numPosSample*OPT.BG_BB_NUM_PIM)) ;

SPMdescFG(OPT.ForeK+1).desc = LSptHist(:,indBackAll) ;
SPMdescFG(OPT.ForeK+1).imgID = single(selImgInd(indSptHist(indBackAll,1))) ;
SPMdescFG(OPT.ForeK+1).bbID = single(indSptHist(indBackAll,2)') ;
%}

% (METHOD 2)


% % --------------------------------------------------------------%
%   GMMdescFG
% % --------------------------------------------------------------%

% For chosen BBs of SPMdescFG, we concatenate img, BB, FG indices.
imgInd = horzcat(SPMdescFG(:).imgID) ;
bbInd = horzcat(SPMdescFG(:).bbID) ;
fgInd = repmat(OPT.ForeK+1, [1 length(imgInd)]) ;
sind = 0 ;
for i=1:OPT.ForeK,
    fgInd(sind+1:sind+length(SPMdescFG(i).imgID)) = i ;
    sind = sind + length(SPMdescFG(i).imgID) ;
end


% find unique chosen images (for fast computation),
[uImgInd, ~, origImgInd] = unique(imgInd) ;

% initialize
GMMdescFG = [] ;
for i=1:OPT.ForeK+1,
    GMMdescFG(i).desc = [] ;
    GMMdescFG(i).cnt = [] ;
    GMMdescFG(i).imgID = [] ;
end

% for each unique chosen image,
for i=1:length(uImgInd),

    % load superpixels & segment
    load (Lfsegm{uImgInd(i)}, 'imgSP', 'imgSeg') ;

    % load SIFT feature
    load(Lffeat{uImgInd(i)}, 'BBsegm', 'segm_bb', 'descSP') ; % , , 'sptHistHSV',

    % find out which BBs come from the i-th unique image
    ifInd = find(imgInd==uImgInd(i)) ;

    % for each BB,
    for j=1:length(ifInd),
        % the coordinates of current BB
        jCoordBB = segm_bb(bbInd(ifInd(j)),:) ;
        % find out which segments are included the chosen BB
        tmpInc = BBsegm - repmat(jCoordBB, [size(BBsegm,1) 1]) ;
        incSeg = find(tmpInc(:,1)>=0 & tmpInc(:,2)>=0 &  tmpInc(:,3)<=0 &  tmpInc(:,4)<=0) ;

        % find out which SPs are included the chosen BB
        incSP = vertcat(imgSP.SPIndPSeg{incSeg}) ;

        % the FG id of current BB
        jFG = fgInd(ifInd(j)) ;
        % update hte descriptor info.
        GMMdescFG(jFG).desc = [GMMdescFG(jFG).desc; descSP.RGB(incSP,:)] ;
        GMMdescFG(jFG).cnt = [GMMdescFG(jFG).cnt; imgSP.SPMass(incSP,:)] ;
        GMMdescFG(jFG).imgID = [GMMdescFG(jFG).imgID imgInd(ifInd(j))] ;
    end

    %.imgID

end


end
