function ImgSeg = get_segments_from_image (flimg, SP, OPTS)
%
%  Generate superpixel from each image
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%   flimg :         input image path
%   SP :            superpixel info
%       .numSP :    number of SPs.
%       .flimg :    image path
%       .numSP :    number of superpixels
%       .imgSP :    superpixel images,
%       .adjMat :   Neighborhood matrix btw segments
%       .cntSeg :   center point of each segment.
%       .massSP :   number of pixels per superpixel ([1 x numSP])
%   OPT :           parameters
%
% --------
% OUTPUTS:
%   ImgSeg :
%       .cntSP :        segment centers
%       .segID :        segment ID of each superpixel
%       .segIDSet :     segment ID of each superpixel (in the form of cell)
%       .adjMat :       adjacency graph between segments
%
% ---------------
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%


img = fn_eql_imsize(imread(flimg),SP.imgSP) ;

if size(img,3)==1,
    % disp('color image only.') ; ImgSeg =[] ; return ;
    disp('gray image!'); img = repmat(img, [1 1 3]) ;
end


% Extract color features from each SP
% descSpCLR: RGB, HSV, or CIELab descriptor of super-pixel
[descSpCLR descSpTEX] = extract_feat_segment(img, double(SP.imgSP));

%construct intra-image network of each image.
[locNet Z_SP] = gen_image_graph(descSpCLR, descSpTEX,...
        SP.cntSeg, OPTS.C_INTRA_NGH, OPTS.WGT_SCALE_PCT, OPTS.CONST_Z) ;

% generate evaluation points (where sources can be placed).
[EvalSet aggloClID] = gen_eval_list(descSpCLR, locNet, ...
        double(SP.imgSP), SP.adjMat, OPTS.NUM_EVAL_PNT, OPTS.CLUST_MIN) ;

%{
% (DEBUG) display the evaluation points.
if bDRAW,
    [label_img] = disp_draw_intra_agglo_real(img, imgSP, EvalSet, 1) ;
    [clusterImg] = disp_get_cluster_img(imgSP, aggloClID) ;
    [ImgSegm] = disp_draw_segment(clusterImg, bDRAW) ;
end
if bSave, imwrite(label_img, [outfpre '_sp_eval' '.png']) ; end
%}

% clear descSpCLR descSpTEX cntSeg adjMat ;

% % --------------------------------------------------------------%
%       Run Diversity Ranking and Clustering
% % --------------------------------------------------------------%

% get diverse and central rank set.
% I implemented two different ways
% (1) Lazy greedy

[segm_cnt segm_cnt_val] = run_div_rank_lazy_greedy(locNet, OPTS.K, ...
        OPTS.LIN_MODE, Z_SP, SP.massSP, EvalSet, OPTS.AUTOK) ;

%{
% (2) naive greedy
[segm_cnt segm_cnt_val] = run_div_rank_naive_greedy(locNet, OPTS.K, ...
        OPTS.AUTOK, OPTS.LIN_MODE, Z_SP, EvalSet, massSP) ;
%}
K = length(segm_cnt) ;

% run clustering from rank set.
[segmentID] = run_cluster_from_rankset(locNet, segm_cnt(1:K), 1:K) ;
K = max(segmentID) ;

% get cluster image
% [segmentImg] = disp_get_cluster_img(SP.imgSP, segmentID) ;

% Get connectivity matrix between segments ([K x K])
segmentID = [0; segmentID] ;
segm_adjmat = mex_adjacency_graph_zero(segmentID(SP.imgSP+1), K) ;
segmentID(1) = [] ;
segm_adjmat = segm_adjmat + segm_adjmat' ; segm_adjmat(segm_adjmat>0) = 1 ;
segm_adjmat = sparse(tril(segm_adjmat)) ;


% % --------------------------------------------------------------%
%     Save the output
% % --------------------------------------------------------------%
%
% It is possible that len(segm_cnt) ~= number of actual segments.
%
% segment centers
ImgSeg.cntSP = segm_cnt ;
% segment ID of each superpixel
ImgSeg.segID = segmentID ;
ImgSeg.segIDSet = cell(1,K) ;
for k=1:K, ImgSeg.segIDSet{k} = find(segmentID==k); end ;
% segment adjacency matrix
ImgSeg.adjMat = segm_adjmat ;
% final segmented image
% ImgSeg.segmentmg = uint8(segmentImg) ;
end

