function [setCandFg valCandFg mstMLE] = infer_tree_cand( ...
    setCandFg, valCandFg, mstCandFg)
%
%   Infer the most likely tree from candidate sets of K foregrounds
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   setCandFg   : candiate sets ([1 x K] cells)
%   valCandFg   : candidate values ([1 x K] cells)
%   mstCandFg   : minimum spanning trees ([1 x K] cells)
%
% --------
% OUTPUTS:
%
%   setCandFg   : candiate sets ([1 x K] cells)
%   valCandFg   : candidate values ([1 x K] cells)
%   mstMLE      : most probable treecandidate values ([1 x K] cells)
%
% ---------------
% Gunhee Kim
%

% number of FG
K = length(setCandFg) ;
% number of segments
nS = size(mstCandFg{1},1) ;

% backup the candidate set
ori_setCandFg = setCandFg ;

% change the format of setCandFg
for k=1:K,
    tmp_set = cell(1, size(setCandFg{k},1)) ;
    for l=1:size(setCandFg{k},1)
        tmp_set{l} = find(setCandFg{k}(l,:)) ;
    end
    setCandFg{k} = tmp_set ;
end
clear tmp_set ;

% value matrix
A = zeros(nS) ;

% min spanning trees for each FG
k_mst = cell(1, K) ;
for k=1:K,
    % ignore diagonal elements
    k_mst{k} = tril(mstCandFg{k},-1) ;
    % ignore values
    k_mst{k}(k_mst{k}~=0) = 1 ;
end

% sum of values
for k=1:K,
    for i=1:length(setCandFg{k}),
        A(setCandFg{k}{i},setCandFg{k}{i}) = A(setCandFg{k}{i},setCandFg{k}{i}) + ...
            valCandFg{k}(i)*k_mst{k}(setCandFg{k}{i},setCandFg{k}{i}) ;
    end
end


% make symmetric
A = A + A' ;
% maximum spanning tree
mstMLE = fn_kruskal (-A) ;

% error handling (if isolated nodes exist, randomly connect them to
% someone).
isolated_node = find(sum(A,1) + sum(A,2)'==0) ;
nisolated = length(isolated_node) ;
if nisolated>0,
    non_isol_node = 1:size(A,1) ; non_isol_node(isolated_node) = [] ;
    rand_ind = randperm(length(non_isol_node)) ;
    rand_node = non_isol_node(rand_ind(1:nisolated)) ;
    maxV = max(mstMLE(mstMLE~=0)) / 100 ;
    for i=1:nisolated,
        s_ind = sort([rand_node(i) isolated_node(i)]) ;
        mstMLE(s_ind(2),s_ind(1)) = maxV ;
    end
end


bmstMLE = mstMLE ;
bmstMLE(bmstMLE~=0) = 1 ;
bmstMLE = bmstMLE + sparse(diag(ones(size(bmstMLE,1),1))) ;

% find out available candidates. (ie candidates should be subgraphs
% of most probable minimum spanning tree).
valid_cand = cell(1,K) ;
for k=1:K,
    valid_cand{k} = zeros(1,length(valCandFg{k}));
    for i=1:length(setCandFg{k}),
        bSub = find( bmstMLE(setCandFg{k}{i},setCandFg{k}{i}) - ...
            k_mst{k}(setCandFg{k}{i},setCandFg{k}{i}) == -1) ;
        if isempty(bSub),
            valid_cand{k}(i) = 1 ;
        end
    end
end


% output 1: candidate sets
for k=1:K,
    setCandFg{k} = ori_setCandFg{k}(valid_cand{k}>0,:) ;
end

% output 2: candidate values
for k=1:K,
    valCandFg{k} = valCandFg{k}(valid_cand{k}>0) ;
end


end
