function [FG_GMM] = run_GMM_FG_init (descFG, FG_Ncomp)
%
%  Get initial GMM foreground models
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   descFG      : Descriptors of foreground pixels for GMM (struct:[1 x ForeK+1])
%                 K+1 foreground == background
%       .desc   : The RGB values
%       .cnt    : the weights (pixels) of each RGB value
%       .imgID  : image ID
%   FG_Ncomp    : number of components of GMM
%
% --------
% OUTPUTS:
%
%   FG_GMM      : GMM foreground model
%       .Ncomp  : number of components
%       .means  : means ([dimFeat x FG_Ncomp])
%       .covs   : covariance ([dimFeat x dimFeat x FG_Ncomp])
%       .sqdetcovs : square of determinant of covariance ([1 x FG_Ncomp])
%       .invcovs: inverse of covariance ([dimFeat x dimFeat x FG_Ncomp])
%       .priors : prior ([1 x FG_Ncomp])
%
% ---------------
% Gunhee Kim
%


% Foreground model: # of components
% initialization
FG_GMM = [] ;
nDim = size(descFG(1).desc,2) ;
for i=1:length(descFG),
    FG_GMM(i).Ncomp = FG_Ncomp ;
    FG_GMM(i).means = zeros(nDim, FG_Ncomp) ;
    FG_GMM(i).covs = zeros(nDim, nDim, FG_Ncomp) ;
    FG_GMM(i).sqdetcovs = zeros(1, FG_Ncomp) ;
    FG_GMM(i).invcovs = zeros(nDim, nDim, FG_Ncomp) ;
    FG_GMM(i).priors = zeros(1, FG_Ncomp) ;
end


% initial GMM model.
% for each foreground
for i=1:length(descFG),

    % descriptors and its counts
    descFG(i).desc = double(descFG(i).desc) ;
    descFG(i).cnt = double(descFG(i).cnt) ;

    % for whatever reasons, errors occur in kmean
    eIter = 0 ;
    while 1,
        try
        % First perform 'kmeans' to detect components
        [compID, compCnt] = kmeans(descFG(i).desc, FG_Ncomp);
        catch,
            disp('Kmean error') ;
            if eIter>3, break; end
            eIter = eIter + 1 ;
            continue ;
        end
        % if successfully done, break while
        break ;
    end

    % for each component
    for j=1:FG_GMM(i).Ncomp,
        tmp_ind = find(compID==j);
        if isempty(tmp_ind), continue ; end

        % weighted prior
        % number of pixels that belong to each component
        FG_GMM(i).priors(j) = sum(descFG(i).cnt(tmp_ind)) ;

        % weighted mean (of descriptors)
        FG_GMM(i).means(:,j) = fn_wmean(descFG(i).desc(tmp_ind,:), ...
                descFG(i).cnt(tmp_ind)) ;
        % weighted cov (of descriptors)
        FG_GMM(i).covs(:,:,j) = fn_wcov(descFG(i).desc(tmp_ind,:), ...
                descFG(i).cnt(tmp_ind)) + ...
                1E-5.*diag(ones(nDim,1)) ;
        %{
        FG_GMM(i).covs(:,:,j) = cov(descFG{i}(tmp_ind,1:end-1)) + ...
            1E-5.*diag(ones(nDim,1)) ;
        %}
        FG_GMM(i).sqdetcovs(j) = sqrt(det(FG_GMM(i).covs(:,:,j))) ;
        if FG_GMM(i).sqdetcovs(j)==0, FG_GMM(i).sqdetcovs(j) = 1 ; end
        FG_GMM(i).invcovs(:,:,j) = inv(FG_GMM(i).covs(:,:,j)) ;
    end
    % divided by total number of pixels
    FG_GMM(i).priors = FG_GMM(i).priors / sum(FG_GMM(i).priors) ;
end

end
