function [optSlnCand prunSetCandFg prunValCandFg] = run_region_assignment_dp( ...
            setCandFg, valCandFg, mstMLE)
%
%   Perform region assignment using dynamic programming
%
% -------
% INPUTS:
%  (Note that there are several other parameters at the top of the code)
%
%   setCandFg       : candiate sets ([1 x K] cells)
%   valCandFg       : candidate values ([1 x K] cells)
%   mstMLE          : MLE minimum spanning tree ([M x M])
%
% --------
% OUTPUTS:
%
%   optSlnCand      : optimal selection of FG candidates ([B x 2])
%                     (FG_ID, Bid ID)
%   prunSetCandFg   : prunned setCandFg
%   prunValCandFg   : prunned valCandFg
%
% ---------------
% Gunhee Kim
%

% K: # of FGs
K = length(setCandFg) ;

% if any same candidates are submitted by more than two foreground,
% we just quickly decide the winnter.
[prunSetCandFg, prunValCandFg] = winner_for_identical_bids(setCandFg, valCandFg, K) ;

% indices of setCandFg ([2 x # of candidates])
% 1st row: foreground ID, 2nd row: local indices
Kid = [] ;
for k=1:K,
    Kid = [Kid [k*ones(1,size(prunSetCandFg{k},1)); 1:size(prunSetCandFg{k},1)]] ;
end


% make them into one
setCandFg = vertcat(prunSetCandFg{:}) ;
valCandFg = horzcat(prunValCandFg{:}) ;

% number of bids
n = length(valCandFg) ;
% number of items
m = size(mstMLE, 1) ;

% mstMLE
mstMLE(mstMLE~=0) = 1 ;
mstMLE_lw = mstMLE ;
mstMLE = mstMLE + mstMLE' ;
mstMLE = mstMLE + eye(m) ;

% get level of each node from mstMLE
level = get_level_nodes(mstMLE) ;
% root where level = 0.
root_id = find(level==0) ;
% sort by level in a descending order
[~, node_dec_lev] = sort(level, 'descend') ;

% get the level of bids (the smallest level of any item in b)
tmp_level = repmat(level,[n 1]) ;
tmp_level(setCandFg==0) = 1000; % large value.
level_b = min(tmp_level,[],2) ;

% In a descending order of levels
opt_IN = cell(1,m) ;
opt_val = zeros(1,m) ;
for i=1:m,

    % current node
    node_i = node_dec_lev(i) ;

    % C_i denotes the set of bids that include node_i and
    % whose level is the same as the level of i.
    C_i = find(level_b==level(node_i)) ;
    C_i = C_i(setCandFg(C_i,node_i)>0) ;
    nC_i = length(C_i) ;

    % The max of the first term in DP equation.
    max_val_C_i = 0 ;
    max_C_i = [] ;
    % valC_i = zeros(nC_i,1) ;
    for c=1:nC_i,

        % bid in consideration
        c_bid = find(setCandFg(C_i(c),:)>0) ;

        % Removing all items of b (c_bid) will decompose
        % the mstMLE into several subtrees. Let them be Tx.
        sub_node = 1:m ; sub_node(c_bid) = [] ;
        mstMLE_c = mstMLE(sub_node, sub_node) ;
        Tx = fn_find_conn_comp(mstMLE_c) ;
        len_Tx = length(Tx) ;

        % remove the subtree that has the root.
        Tx_root = [] ;
        for t=1:len_Tx,
            Tx{t} = sub_node(Tx{t}) ;
            if sum(Tx{t}==root_id)>0, Tx_root = t; end
        end
        Tx(Tx_root) = [] ;
        len_Tx = length(Tx) ;

        % Ub: the roots of Tx
        Ub = zeros(1,len_Tx) ;
        for t=1:len_Tx,
            % the root of Tx is the note whos opt_val is the largest.
            [max_val max_ind] = max(opt_val(Tx{t})) ;
            if max_val==0, continue; end
            Ub(t) = Tx{t}(max_ind) ;
        end

        valC_c = valCandFg(C_i(c)) + sum(opt_val(Ub)) ;

        if valC_c > max_val_C_i,
            max_val_C_i = valC_c ;
            max_C_i = [C_i(c) horzcat(opt_IN{Ub})] ;
        end
    end

    % second term of DP equation
    % first find the children of node_i
    child_i = [find(mstMLE_lw(node_i,:)==1) find(mstMLE_lw(:,node_i)'==1)] ;
    % if its value is 0, it is not a child.
    child_i(opt_val(child_i)==0) = [] ;

    child_val = sum(opt_val(child_i)) ;
    child_i = horzcat(opt_IN{child_i}) ;

    if max_val_C_i > child_val,
        % backup the optimum
        if ~isempty(find(sum(setCandFg(unique(max_C_i),:),1)>1)),
            disp('error 1 in run_region_assignment_dp.m'); keyboard ;
        end
        opt_val(node_i) = max_val_C_i ;
        opt_IN{node_i} = unique(max_C_i) ;
    else
        if ~isempty(find(sum(setCandFg(unique(child_i),:),1)>1)),
            disp('error 2 in run_region_assignment_dp.m'); keyboard ;
            disp('err');
        end
        opt_val(node_i) = child_val ;
        opt_IN{node_i} = unique(child_i) ;
    end
end


% optimum region assignemtn
optSlnCand = Kid(:,opt_IN{level==0})' ;

if find(sum(setCandFg(opt_IN{level==0},:))>1)
    disp('opt error: error1') ;
end
if find(sum(setCandFg(opt_IN{level==0},:))==0)
    disp('opt error: error2') ;
end

end


% get bid graph
function [G] = get_bid_graph(setCandFg)

[nB nS] = size(setCandFg) ;

G = sparse(nB, nB) ;

for i=1:nS,
    shared_ind = find(setCandFg(:,i)>0) ;
    len_shared = length(shared_ind) ;
    for s=len_shared:-1:2,
        G(s,shared_ind(1:s-1)) = 1 ;
    end
end

end


% get level of each node from mst
function [level] = get_level_nodes(mst)

m = size(mst,1) ;

% minimum spanning tree (only connectivity)
%mst(mst>0)=1 ;
%mst = mst + mst' ;

% root = the node with maximum edges.
[foo root] = max(sum(mst)) ;

% the level of root = 0 ;
level = -1*ones(1,m) ;
level(root) = 0 ;

% search children of parents, and set the level
parent = root ;
p_lev = 0 ;
while ~isempty(find(level==-1)),

    p_lev = p_lev + 1 ;

    [foo child] = find(mst(parent,:)) ;
    child = unique(child) ;
    child = child(level(child)==-1) ;

    level(child) = p_lev ;

    parent = child ;

end

end

% if any candidate is submitted via more than two foreground,
% we just keep the best one.
function [setCandFg, valCandFg] = winner_for_identical_bids(setCandFg, valCandFg, ForeK) ;

% if any candidate is submitted by more than two foreground,
% we just keep the best one.
for k=1:ForeK-1,
    for l=k+1:ForeK,
        % find overlapped elements
        [~, ik, il] = intersect(setCandFg{k}, setCandFg{l}, 'rows') ;

        % find which one has the smaller value and remove it
        [~, min_id] = min([valCandFg{k}(ik); valCandFg{l}(il)]) ;

        % find the removed indices
        del_ind_k = ik(min_id==1) ;
        del_ind_l = il(min_id==2) ;

        setCandFg{k}(del_ind_k,:) = [] ;
        setCandFg{l}(del_ind_l,:) = [] ;

        valCandFg{k}(del_ind_k) = [] ;
        valCandFg{l}(del_ind_l) = [] ;

    end
end

clear foo ik il max_id del_ind_k del_ind_l ;

end