function script_oversegment (conf_file, strObj)
%
% Oversegment images.
% We implement two different methods (1) Normalized cuts, and (2) Submodular segmentation
% (Note that there are several other parameters at the top of the code)
% -------
% INPUTS:   NONE.
%
% --------
% OUTPUTS:
%
%   This code will save two struct 'imgSeg' and 'imgSP'.
%
%   imgSeg          : Oversegment info
%       .imSize     : original image size
%       .flimg      : image path
%       .segImg     : segment image
%       .adjMat     : adjacency matrix between segments
%
%   imgSP           : superpixel(SP) info
%       .SPImg      : SP image
%       .SPcnt      : center positino of each SP
%       .SPMass     : # of pixels of each SP
%       .SP2Seg     : mapping btw SP and segments
%       .SPIndPSeg  : superpixel indices of each segment
%
% --------
% DEPENDENCY:
%   if you set OPT_SEG.TYPE==2, you require 'subopt_anidiff' package
%   Otheriwse, with OPT_SEG.TYPE==1, you require 'normalized cuts' package
%
% ---------------
% Written by Gunhee Kim @ CMU, 2012.
% All rights reserved.
%

% run config file
if nargin<1,
    conf_file = 'config_labelme' ;
    conf_file = 'config_flickr' ;
end
eval(conf_file) ;

% For condor
if nargin>1,
    CLASSES{1} = strObj; NCLASS = 1 ;
end


% % ------------------------------------------------------------------------%
% %     Main function
% % ------------------------------------------------------------------------%

% for each object,
for h=1:NCLASS,
    get_oversegment(conf_file, CLASSES{h}) ;
end

% exit

end


% run oversegmentation
function get_oversegment(confFile, obj_name)

% run config file
eval(confFile);

% % ------------------------------------------------------------------------%
% %     Parameters
% % ------------------------------------------------------------------------%

% (1) Normalized cuts
if OPT_SEG.TYPE==1,
    % resizing images for fast segmentation
    OPT_SEG.MAX_IMG_SIZE = 180 ;
end

% folders
DIR_WORK_IMG = fullfile(DIR_WORK_IMG, obj_name) ;
DIR_D_INFO = fullfile(DIR_WORK_TRAIN, FOLDR_D_INFO, obj_name) ;
DIR_WORK_SP = fullfile(DIR_WORK_TRAIN, ...
    [FOLDR_SUPERPIXELS num2str(OPT_SEG.NUM_SP)], obj_name) ;
DIR_WORK_SEGM = fullfile(DIR_WORK_TRAIN, [FOLDR_SEGM ...
    num2str(OPT_SEG.NUM_SP) '_' num2str(OPT_SEG.K)], obj_name) ;
if ~exist(DIR_WORK_SEGM,'dir'), mkdir(DIR_WORK_SEGM) ; end


% load file DB
DIR_D_INFO = fullfile(DIR_WORK_TRAIN, FOLDR_D_INFO, obj_name) ;
DIn = fn_get_fildDB(DIR_D_INFO, FNAME_D_INFO) ;
DAll = [] ;
for i=1:length(DIn),
    load(fullfile(DIR_D_INFO, DIn(i).folder, DIn(i).fname), 'D') ;
    DAll = [DAll D] ;
end
D = DAll ; DAll = [] ;
nD = length(D) ;


warning off ; tic ;
% for each image
for i=1:nD,
    % input and output file names
    fImg = fullfile(DIR_WORK_IMG, D(i).folder, D(i).fname) ;
    dSegm = fullfile(DIR_WORK_SEGM, D(i).folder) ;
    fSegm = fullfile(dSegm, strrep(D(i).fname, TYPE_IMG, TYPE_MAT));
    % fSP = fullfile(DIR_WORK_SP, D(i).folder, strrep(D(i).fname, TYPE_IMG, '_SP.mat'));

    % make directory
    if ~exist(dSegm, 'dir'), mkdir(dSegm) ; end

    % if already done, skip it.
    % if exist(fSegm, 'file'), continue; end

    % (1) Normalized cuts
    if OPT_SEG.TYPE==1,
        % run Normalized cuts
        try
            [img imSize] = imread2(fImg, OPT_SEG.MAX_IMG_SIZE, 1) ;
        	segment = ncut_multiscale(img, NSEGS);
            segment = uint8(segment) ;
        catch, continue ; end

        % save the results
        imgSeg.imSize = imSize ;
        imgSeg.fImg = fImg ;
        imgSeg.segImg = segment ;

        save(fSegm, 'imgSeg') ;
    % (2) Gunhee's ICCV 2011
    else
        % Set options
        % see the default options in 'opts_single_seg.m'
        OPTS = opts_single_seg(OPT_SEG) ;

        % run superpixeling
        % Note that TurboPixel requires IP toolbox.
        SP = get_superpixel_from_image (fImg, OPTS.MAX_IMG_SIZE, ...
            OPTS.NUM_SP, OPTS.SP_MODE) ;
        % save output
        % save(fSP, 'SP') ;
        % load(fSP, 'SP') ;

        % run oversegmentation
        Seg = get_segments_from_image(fImg, SP, OPTS) ;

        % Informaton about image segmentation
        imgSeg.imSize = SP.imSize ;
        imgSeg.flimg = SP.flimg ;
        imgSeg.segImg = get_segm_img_from_sp(SP.imgSP, Seg.segID) ;

        % Information about superpixels
        imgSP.SPImg = SP.imgSP ;
        imgSP.SPCnt = single(SP.cntSeg) ;
        imgSP.SPMass = single(SP.massSP) ;
        imgSP.SP2Seg = single(Seg.segID) ;

        % SPs indices per segment
        numSeg = max(imgSP.SP2Seg) ;
        imgSP.SPIndPSeg = cell(1, numSeg) ;
        for j=1:numSeg,
            imgSP.SPIndPSeg{j} = single(find(imgSP.SP2Seg==j)) ;
        end

        % adjacency matrix between segments
        imgSeg.adjMat = get_adjency_matrix(imgSeg.segImg) ;

        %{
        % (DEBUG) Draw Superpixels
        imgSPClr = disp_draw_segment_by_color(imgSP.SPImg) ;
        imgSPHsv = disp_draw_hsv_segment_on_img(fImg, ...
            imgSP.SPImg) ;
        figure; subplot(1,2,1); imshow(imgSPClr) ;
        subplot(1,2,2); imshow(imgSPHsv) ;

        % (DEBUG) Draw Segmentation
        imgSegClr = disp_draw_segment_by_color(imgSeg.segImg) ;
        imgSegHsv = disp_draw_hsv_segment_on_img(fImg, ...
            imgSeg.segImg) ;
        figure; subplot(1,2,1); imshow(imgSegClr) ;
        subplot(1,2,2); imshow(imgSegHsv) ;
        keyboard;
        %}
disp('ver') ;
keyboard;
        % save output
        save(fSegm, 'imgSeg', 'imgSP') ;
    end

    % display the progress
    if mod(i,10) == 0,
        t2 = toc ; tic ;
        disp([num2str(i) ' out of ' num2str(nD) ' : ' num2str(t2) ' sec.']) ;
    end
end

disp(['Oversegmenting ' obj_name ' is done.']) ;

end
