function [Lrank_set Lrank_set_val] = run_coseg_naive_greedy(...
            Ladj_mat, K, lin_mode, Z, LEvalList, LmassPnt, InterNet)
%
%   Run co-segmentation using naive greedy method.
%
% -------
% INPUTS:
%
%	Ladj_mat    : The cell of similarity networks (M x [N x N])
%                 where M: # of images, N: # of vertices.
%   K           : number of ranked sets
%   lin_mode    : methods to solve linear eq Ax=b.
%                 0: incremental inverse. 1: LU decomposition.
%                 Others: naive backslash.
%   Z           : ground conductance (M x[N x 1])
%   LEvalList   : The cell of the vertices that are evaluated to search for max gain.
%                 As default, we evaluate all vertices.
%   LmassPnt    : mass (weights) of each point (M x [N x 1])
%   InterNet    : Inter-network across the SPs of images.
%    .num_eval  : number of evaluation points each image ([1 x M])
%    .mat       : similarity matrix between eval points of
%                 different images ([M x M] cells)
% --------
% OUTPUTS:
%
%   rank_set        : selected rank_sets ([1 x K])
%   rank_set_val    : potentials ([1 x K])
%
% -------------
%
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%

num_set = length(Ladj_mat) ;

% number of vertex
for h=1:num_set
    num_vtx{h} = size(Ladj_mat{h},1) ;
end

% default parameters
if nargin<5,
    LEvalList = cell(1,num_set) ;
    for h=1:num_set,
        LEvalList{h} = 1:num_vtx{h} ;
    end
end
if nargin<6,
    LZ = cell(1,num_set) ;
    for h=1:num_set,
        LZ{h} =  0.001*ones(num_vtx{h},1) ;
    end
end
if nargin<7,    lin_mode = 0 ;          end

% number of evaluation points.
nLEvalList = cell(1,num_set) ;
for h=1:num_set,
    nLEvalList{h} = length(LEvalList{h}) ;
end

% Add dummy ground and normalize it.
Lneg_pole = cell(1,num_set) ;
Lind_ord = cell(1,num_set) ;
for h=1:num_set
    Ladj_mat{h} = [[Ladj_mat{h} Z{h}]; [ones(1,num_vtx{h}) 0]] ;
    Ladj_mat{h} = fn_normout(Ladj_mat{h}) ;
    Lneg_pole{h} = num_vtx{h} + 1 ;
    Lind_ord{h} = 1:num_vtx{h} ;
end


% output
Lrank_set = cell(1,num_set) ;
Lrank_set_val = cell(1,num_set) ;
for h=1:num_set
    Lrank_set{h} = [] ; Lrank_set_val{h} = [] ;
end

% Greedily select K-1 more nodes
% Picking out the node causing max potential increase one by one.
% Once picked out, the item turns into voltage sources.
for i=1:K,
    tic
    obj_val = cell(num_set,1) ;
    for h=1:num_set,
        % Get the sets to be evaluated.
        pos_set = sparse(LEvalList{h}, 1:nLEvalList{h}, ones(1,nLEvalList{h}), num_vtx{h}+1, nLEvalList{h}) ;
        pos_set(Lrank_set{h},:) = 1 ;
        neg_set = Lneg_pole{h} ;
        % (1) incremental inverse matrix
        if lin_mode==0,
            potential = get_potential_mult_lineq_inc_inv(Ladj_mat{h}, pos_set, neg_set) ;
        % (2) LU decomposition
        elseif lin_mode == 1,
            potential = get_potential_mult_lineq_lu(Ladj_mat{h}, pos_set, neg_set) ;
        % (3) Naive method using backslash.
        else
            potential = get_potential_mult_lineq_naive(Ladj_mat{h}, pos_set, neg_set) ;
        end
        clear pos_set neg_set ;

        % potential of SP = computed potential * number of pixels of SP.
        potential(Lind_ord{h},:) = potential(Lind_ord{h},:) .* ...
            repmat(LmassPnt{h}, [1 nLEvalList{h}]) ;

        obj_val{h} = sum(potential) ;
    end
    [tv ti] = max(obj_val{h}) ;

    %{
    %keyboard;
    unit_obj_val = obj_val{2} ;
    unit_InterNet = InterNet.mat{2,3} ;
    unit_InterNet = [unit_InterNet,InterNet.mat{2,3}(:,end)] ;

    for h=1:1000,
        obj_val{h} = unit_obj_val ;
    end

    InterNet.num_eval = length(unit_obj_val) * ones (1,1000) ;
    tic
    for h=1:1000,
        for hh=1:1000,
            if h==hh, continue; end
            InterNet.mat{h,hh} = unit_InterNet ;
        end
    end
    toc
    %}

    % run bp to get max sources
    [max_label] = run_bp_max_sources (obj_val, InterNet) ;

    % save the vertex to maximuze the marginal temperature gain.
    for h=1:num_set
        Lrank_set{h} = [Lrank_set{h}; LEvalList{h}(max_label(h))] ;
    end

    % display the result
    t2 = toc ;
    %str_rank_set = [];
    %for j=1:length(rank_set),
    %    str_rank_set = [str_rank_set num2str(rank_set(j)) ' '] ;
    %end
    display([num2str(i) ' sources are chosen. ' ...
        num2str(t2) ' sec.']) ;
end

end

% run bp to get max sources
% obj_val   : object values
% InterNet  : Inter-network across the SPs of images.
function [max_label_new] = run_bp_max_sources (obj_val, InterNet)

    % iteration of bp.
    ITER = 10 ;

    % number of images
    num_set = length(obj_val) ;

    % Initialize message
    nLEvalList = zeros(num_set, 1) ;
    for i=1:num_set,
        nLEvalList(i) = length(obj_val{i}) ;
    end

    % Initialize message
    msg_old = cell(num_set, 1) ;
    for i=1:num_set,
        msg_old{i} = zeros(nLEvalList(i), num_set) ;
    end
    msg_new = msg_old ;

    max_label_old = zeros(num_set,1) ;
    max_label_new = zeros(num_set,1) ;

    for hh=1:ITER,
        % update message
        for i=1:num_set,
            for j=1:num_set,
                if i==j,    continue;
                elseif i<j, locIntNet = InterNet.mat{i,j} ;
                else        locIntNet = InterNet.mat{j,i}' ;
                end

                %{
                % compute message from j to i.
                for k =1:size(msg_old{i},1),
                    msg_new{i}(k,j) = max(obj_val{i}(k)*locIntNet(k,:)' + ...
                        sum(msg_old{j}(:,[1:i-1 i+1:end]),2)) ;
                end
                %}

                % the above for loop is the same with the following.
                % compute message from j to i.
                msg_new{i}(:,j)= max(repmat(obj_val{i},[nLEvalList(j) 1]) .* locIntNet' + ...
                    repmat(sum(msg_old{j}(:,[1:i-1 i+1:end]),2),[1 nLEvalList(i)]))' ;
            end
        end

        % labeling updated
        for i=1:num_set,
            [tmp_max_v tmp_max_i] = max(sum(msg_new{i},2)) ;
            max_label_new(i) = tmp_max_i ;
        end

        % if the label is unchanged, break it.
        if sum(abs(max_label_new - max_label_old))==0,
            break;
        end
        % update old <- new.
        max_label_old = max_label_new ;
        msg_old = msg_new ;
    end
end
