function [potentials] = get_potential_mult_lineq_power(cond_mat, ...
                        pos_set, neg_pole)
%
%	Compute Potentials for multiple sets
%   Solution by incremental inverse
%   Refer to http://en.wikipedia.org/wiki/Block_LU_decomposition
%
% -------
% INPUTS:
%
%	cond_mat        : conductance network
%   pos_set         : 1 if nodes are attached to sources. Otherwise, 0
%                     [N x M] where N: # of nodes, M: # of sets
%                     Each set is assumed to have the same number of positive poles
%                     except the already selected vertices
%   neg_pole        : nodes are attached to ground ([1 x L])
%                     where L is the number of negative nodes.
%
% --------
% OUTPUTS:
%   potentials      : potential  ([N x M])
%
% -------------
%
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%

% N: number of vertices
% M: number of sets
[N M] = size(pos_set) ;

% tolerance for sum
CONST_TOL = M * 10^-5 ;

% number of iterations for maximum voltage increase
CONST_ITER_VOL = 50000 ;

% if neg_pole is not assigned, report an error.
if isempty(neg_pole)
    error('Negative pole should be given.');
end

% if the matrix is not normalized.
if abs(sum(cond_mat(:))-N)>0.0001*N,
    cond_mat = fn_normout(cond_mat);
end

% Positive poles shared by all sets
shared_pos_pole = find(sum(pos_set,2)==M)' ;

% initial potentials
if isempty(shared_pos_pole),
    v = full(pos_set) ;
% if there's shared positive poles, use the potential with them as
% initial voltage.
else
    v = get_potential_lineq(cond_mat, [], shared_pos_pole, N) ;
    v = repmat(v, [1 M]) ;
    v(pos_set==1) = 1 ;
end

% avail indices
avail_ind = 1:M ;

% outputs
potentials = zeros(N, M) ;
% previous potential sum
pre_sum_v = sum(v) ;
for i=1:CONST_ITER_VOL,

    % power iteration
    v = cond_mat * v ;
    v(pos_set==1) = 1 ;
    v(end,:) = 0 ;

    % compute potential sum
    sum_v = sum(v) ;
    % compute the potential difference
    diff_v = sum_v - pre_sum_v ;
    pre_sum_v = sum_v ;

    % if the potential increase is within the tolerance
    conv_ind = find(diff_v < CONST_TOL) ;
    % back up the potentail and remove them.
    if ~isempty(conv_ind),
        potentials(:, avail_ind(conv_ind)) = v(:,conv_ind) ;
        v(:,conv_ind) = [] ;
        avail_ind(conv_ind) = [] ;
        pos_set(:,conv_ind) = [] ;
        pre_sum_v(conv_ind) = [] ;
    end
    % all cases are converged, break the loop.
    if isempty(avail_ind),
        break ;
    end
end

end
