function [src_set src_set_val] = run_div_rank_power_greedy(adj_mat, ...
                        K, Z, massPnt, EvalList, defSrc)
%
%   Run diversity ranking using power iteration.
%   Power iteration is useful to quickly find "good" reasonable solutions. 
%   One important issue is that sometimes the convergence may be
%   slow, so there is a trade-off between speed and exact search. 
%   In practice, the solution by this method is good enough. 
%
% -------
% INPUTS:
%
%	adj_mat :           Similarity network ([N x N]) 
%                       where N is the number of vertices 
%   K :                 number of ranked sets
%   RATIO_EXACT_COMP :  Ratio of nodes for exact potential computation. ([1 x 1])
%   Z :                 ground conductance ([N x 1]) 
%   massPnt :           mass (weights) of each point ([N x 1])
%                       (default: ones(N,1)).
%   EvalList :          The vertices that are evaluated to search for max gain.
%                       As default, we evaluate all vertices. 
%   defSrc :            The default sources
%
% --------
% OUTPUTS:
%
%   src_set :           selected sourse sets ([1 x K]) 
%   src_set_val :       their potentials ([1 x K]) 
%
% -------------
%
% Written by Gunhee Kim @ CMU, 2011.
% All rights reserved.
%

% number of vertex
num_vtx = size(adj_mat,1) ;

% default parameters
if nargin<3 || isempty(Z),                  Z = 0.001*ones(num_vtx,1) ; end
if nargin<4 || isempty(massPnt),            massPnt = ones(num_vtx,1) ; end
if nargin<5 || isempty(EvalList),           EvalList = 1:num_vtx ;      end
if nargin<6 || isempty(defSrc),             defSrc = [] ;               end

% mininum number of exact search
MIN_N_EXACT_SEARCH = 1 ;
% maximum number of exact search
MAX_N_EXACT_SEARCH = 20 ;
% ratio of exact search
RATIO_EXACT_COMP = 0 ;

% Method for exact potential computation
% lin_mode : methods to solve linear eq Ax=b.
% 0: incremental inverse. 1: LU decomposition.
% 2: naive backslash. Others: power iteration.
lin_mode = 0 ;     

% number of evaluation points.
nEvalList = length(EvalList) ;

% Add dummy ground and normalize it.
adj_mat = [[adj_mat Z]; [ones(1,num_vtx) 0]] ;
adj_mat = fn_normout(adj_mat) ;
neg_pole = num_vtx + 1 ;
% all indices except negative pole.
ind_ord = 1:num_vtx ;

% output
src_set = defSrc ; src_set_val = [] ;
if ~isempty(defSrc),    
    src_set_val = zeros(1, length(defSrc)) ;
end

% Greedily select K-1 more nodes 
% Picking out the node causing max potential increase one by one.
% Once picked out, the item turns into voltage sources.
for i=1:K,

    iEvalList = setdiff(EvalList, src_set) ;
    niEvalList = length(iEvalList) ;

    tic
    % Get the sets to be evaluated. 
    pos_set = sparse(iEvalList, 1:niEvalList, ones(1,niEvalList), num_vtx+1, niEvalList) ;
    pos_set(src_set,:) = 1 ;
    neg_set = neg_pole ;

    % get the top-(numList) number of best evaluation candidates.
    numList = max(round(RATIO_EXACT_COMP*num_vtx), MIN_N_EXACT_SEARCH) ;
    numList = min(numList, MAX_N_EXACT_SEARCH) ;
    [PrunList PrunPot] = search_maxgain_power_iter(adj_mat, numList, pos_set) ;
    PrunList = iEvalList(PrunList) ;
    nPrunList = length(PrunList) ;

    % Only if nPrunList is larger than 1. Otherwise, just use the
    % result of 'search_maxgain_power_iter.m'.
    if nPrunList>1,
        pos_set = sparse(PrunList, 1:nPrunList, ones(1,nPrunList), num_vtx+1, nPrunList) ;
        pos_set(src_set,:) = 1 ;

        % (1) incremental inverse matrix
        if lin_mode==0,
            [potential] = get_potential_mult_lineq_inc_inv(adj_mat, pos_set, neg_set) ;
        % (2) LU decomposition
        elseif lin_mode == 1,
            [potential] = get_potential_mult_lineq_lu(adj_mat, pos_set, neg_set) ;
        % (3) Naive method using backslash.
        elseif lin_mode == 2,
            [potential] = get_potential_mult_lineq_naive(adj_mat, pos_set, neg_set) ;
        % (4) Power iteration
        else
            [potential] = get_potential_mult_lineq_power(adj_mat, pos_set, neg_pole) ;
        end
    else
        potential = PrunPot ;
    end

    % potential of SP = computed potential * number of pixels of SP.
    potential(ind_ord,:) = potential(ind_ord,:) .* repmat(massPnt, [1 nPrunList]) ;

    %{
    % (DEBUG)
    [potential1] = get_potential_mult_lineq_naive(adj_mat, pos_set, neg_set) ;
    max(max(abs(potential-potential1)))
    %}

    clear pos_set neg_set ;

    % save the vertex to maximuze the marginal temperature gain.
    [max_v max_i] = max(sum(potential)) ;
    max_i = PrunList(max_i) ;
    src_set = [src_set max_i] ;
    src_set_val = [src_set_val max_v] ;

    % display the result
    t2 = toc ;
    str_src_set = []; 
    for j=1:length(src_set),
        str_src_set = [str_src_set num2str(src_set(j)) ' '] ;
    end
    %display(['[ ' str_src_set '] are chosen as sources. ' ...
    %    num2str(t2) ' sec.']) ;

end

%{
% I implemented three different ways to compute potnetials
% (1) power method, (2) Using '\' (3) LU decomposition
potential = get_potential_power(adj_mat, src_set, num_vtx+1) ;
potential = get_potential_lineq(adj_mat, [], src_set, num_vtx+1) ;
potential = get_potential_lineq_lu(adj_mat, [], src_set, num_vtx+1) ;
%}

end
