function [SolList SolV] = search_maxgain_power_iter(adj_mat, numList, pos_set)
%
%	User power iteration to quickly search for the good solution.
%   It's a myopic search within fixed t. (ie. before achieving the
%   steady state). 
%
%   There are some parameters for trade-off between speed and exactness. 
%   For speed, do the following (1) increase MIN_CONST_TOL, or 
%   (2) decrease MIN_ITER or CONST_ITER_VOL.
% -------
% INPUTS:
%
%	adj_mat :       Normalized similarity network. 
%   numList :       Number of solution candidates as output
%   pos_set :       The set of positive poles ([N x M])
%
% --------
% OUTPUTS:
%
%   SolList :       The set of chosen candidate 'pos_set'. 
%   SolV :          The voltage of the SolList.
%
% -------------
%
% Gunhee Kim

% tolerance for termperature increase per iteration
CONST_TOL = 10 ;
MIN_CONST_TOL = 10^-4 ;

% decrease rate for tolerance
DECR_RATE_TOL = 0.8 ; 

% if the marginal gain is less than IGNORE_VOL_RATE*max val, 
% ignore the post_set
IGNORE_VOL_RATE = 0.1 ;

% N: number of vertices
% M: number of sets
[N M] = size(pos_set) ;

% minimum iteration
MIN_ITER = 3*round( N * N / nnz(adj_mat) ) ;
% number of iterations for maximum voltage increase
CONST_ITER_VOL = 5*MIN_ITER ;

% Positive poles shared by all sets
shared_pos_pole = find(sum(pos_set,2)==M)' ;

% output
SolList = [] ; 

% initial potentials
if isempty(shared_pos_pole), 
    potentials = full(pos_set) ;
    init_sum_v = sum(potentials) ;
% if there's shared positive poles, use the potential with them as
% initial voltage.
else
    potentials = get_potential_lineq(adj_mat, [], shared_pos_pole, N) ;
    potentials = repmat(potentials, [1 M]) ;
    init_sum_v = sum(potentials) ;
    potentials(pos_set==1) = 1 ;
end

% initialization
v = potentials ;
pre_sum_v = 10000*ones(1,M) ;
sum_v = sum(v) ;
diff_v = 10000*ones(1,M) ;

% loop
tot_iter  = 0 ;
while 1,
    % available indices
    avail_ind = 1:M ;

    for i=1:CONST_ITER_VOL,

        % ignore the indices whose value is too small compared to the maximum
        % becuase our goal here is to find the point with maximal gain. 
        if mod(i,5)==1 ,
            marginal_v = sum_v(avail_ind) - init_sum_v(avail_ind) ;
            [max_sum_val_avail max_ind_avail] = max(marginal_v) ;
            del_ind = find(marginal_v < IGNORE_VOL_RATE * max_sum_val_avail) ;
            if ~isempty(del_ind),
                avail_ind(del_ind) = [] ;
            end
        end

        % compute the potential difference
        diff_v(avail_ind) = abs(sum_v(avail_ind) - pre_sum_v(avail_ind)) ;

        % if the potential increase is within the tolerance
        conv_ind = find(diff_v(avail_ind) < CONST_TOL) ;

        % back up the potentail and remove them.
        if ~isempty(conv_ind),
            % potentials(:, avail_ind(conv_ind)) = v(:,avail_ind(conv_ind)) ;
            avail_ind(conv_ind) = [] ;
        end

        % all cases are converged, break the loop. 
        if length(avail_ind) < numList,
            break ;
        end

        % power iteration
        v(:,avail_ind) = adj_mat * v(:,avail_ind) ;
        v(pos_set==1) = 1 ;
        v(end,avail_ind) = 0 ;    

        % backup the potential sum
        pre_sum_v(avail_ind) = sum_v(avail_ind) ;
        % compute the potential sum
        sum_v(avail_ind) = sum(v(:,avail_ind)) ;
    end

    % total executions of power iteration
    tot_iter = tot_iter + i - 1 ;

    % disp([num2str(i) ' ' num2str(tot_iter) ' ' num2str(CONST_TOL)]) ;
    % there are three different ways to terminate
    % (1) if the number of single iteration is larger than MIN_ITER
    % (2) if the accumulated iteration is larger than CONST_ITER_VOL
    % (3) If the tolerance is lower than MIN_CONST_TOL
    if i>MIN_ITER, break ; end
    if tot_iter > CONST_ITER_VOL, break ; end
    if CONST_TOL < MIN_CONST_TOL, break ; end
    
    % decrease tolerance
    CONST_TOL = DECR_RATE_TOL*CONST_TOL ;
end

%disp(['In search_evallist_power_iter, Total Iteration = ' ...
%        num2str(tot_iter) ', tol = ' num2str(CONST_TOL)]) ;

% find the points with top-(numList) highest potentials
[sort_val sort_ind] = sort(sum(v), 'descend') ;
SolList = sort_ind(1:min(numList,M)) ;
SolV = v(:,SolList) ;

% figure; plot(1:length(sort_val), sort_val) ;

end
