function [Canvas] = draw_imgs_big_canvas(img_path, bDraw, path_out, img_size, ...
    MAX_NUM_H, h_margin, v_margin) 
%
%	Draw images (whose paths: 'img_path') on a single big canvas ('Canvas').
%   Each of 'img_path' is resized to 'img_size' and placed on a
%   rectangular grid (length(img_path)/MAX_NUM_H X MAX_NUM_H).
%   
%
% -------
% INPUTS:
%
% 	img_path        : The paths of images to be drawn. ROW-WISE
%                     or it can be a cell of image files.
%   bDraw           : Whether the 'Canvas' is shown (1) or not (0).
%   path_out        : If it is not empty, 'Canvas' is saved in 'path_out'.
%   img_size        : The image size of each unit image. ([1 x 3])
%   MAX_NUM_H       : The number of columns.
%   h_margin        : The horizontal space between element images.
%   v_margin        : The vertical space between element images.
%
% --------
% OUTPUTS:
%
%   Canvas          : output image.
%
% -------------
%
%   Gunhee Kim (gunhee@cs.cmu.edu)
%   08/25/2010
%

% (default) do not draw the result.
if nargin<2, 
    bDraw = 0 ;
end
% (default) do not save the result.
if nargin<3,
    path_out = [] ;
end
% (default) number of columns = 20
if nargin<4, 
    MAX_NUM_H = 10 ;
end
% (default) size of each unit image = [256 256 3] 
if nargin<5, 
    img_size = [256 256 3] ;
end
% (default) default horizontal space = 2 
if nargin<6, 
    h_margin = 2 ;
end
% (default) default vertical space = 2 
if nargin<7, 
    v_margin = 2 ;
end

% number of columns and rows.
num_img = length(img_path) ;
MAX_NUM_H = min(MAX_NUM_H,num_img) ;
num_v = ceil(num_img/MAX_NUM_H) ;

% draw big canvas.
Canvas = 255*ones(img_size(1)*num_v+(num_v-1)*v_margin, ...
    img_size(2)*MAX_NUM_H+(MAX_NUM_H-1)*h_margin, 3, 'uint8') ;

max_ch = 0 ;
h_pnt = 0 ;
v_pnt = 0 ;
for i=1:num_img,
    % check whther image path or images.
    if ischar(img_path{i}), 
        im = imread(img_path{i}) ; 
    else
        im = img_path{i} ;
    end

    nch = size(im,3) ;
    if nch>max_ch,  max_ch = nch ; end
    % if the image is gray scale, use 'nearest' option for imresize
    if nch == 1,
        im = imresize(im, [img_size(1),img_size(2)], 'nearest');    
        im = repmat(im, [1 1 3]) ;
    % if it's colored, use 'bilinear' option
    else
        im = imresize(im, [img_size(1),img_size(2)], 'bilinear');
    end

    Canvas(v_pnt+1:v_pnt+img_size(1),h_pnt+1:h_pnt+img_size(2),:) = im ;
    h_pnt = h_pnt + img_size(2) + h_margin ;
    if mod(i,MAX_NUM_H)==0,
        h_pnt = 0 ;
        v_pnt = v_pnt + img_size(1) + v_margin ;
    end
end
% if all images are gray-scale
if max_ch == 1, 
    Canvas = Canvas(:,:,1) ;
end

% show the result
if bDraw>0, 
    figure; imshow(Canvas) ;
end

% save the result
if ~isempty(path_out),
    imwrite(Canvas, path_out) ;
end