function [descSPClr descSPTex] = extract_feat_segment( ...
            img, spImg, clrMode, textMode)
%
%   Extract feature of each segment
%
% -------
% INPUTS:
%  
%   img             : The input image
%   spImg           : Superpixel image 
%   clrMode         : color descriptor mode 
%                     0: No, 1: L*ab, 2: RGB, 3: HSV
%   textMode        : texture descriptor mode
%                     0: No, 1: Yes.
%
% --------
% OUTPUTS:
%
%   descSPClr       : Color descriptor of each segment ([S x 3]).
%   descSPTex       : Texture descriptor of each segment ([S x 4]).
%
% ---------------
% Gunhee Kim
%

% As default, L*ab color and no texture descriptor.
if nargin<3, 
    clrMode = 1 ;
end
if nargin<4,
    textMode = 0 ;
end


% image size
[isz_r isz_c isz_ch] = size(img) ;

% output
descSPClr = [] ; descSPTex = [] ; 

% color descriptor type
switch clrMode,
    case 1      % L*ab
        img_clr = fn_RGB2Lab(img) ;
        img_clr = reshape(double(img_clr), [isz_r*isz_c isz_ch]) ;

    case 2      % RGB
        img_clr = reshape(double(img), [isz_r*isz_c isz_ch]) ;
    case 3      % HSV
        img_clr = fn_rgb2hsv_fast(img,'single') ;
        img_clr = reshape(double(img_clr), [isz_r*isz_c isz_ch]) ;
end

% texture descriptor type
switch textMode,
    case 1
        if size(img, 3)==3
            img_f = rgb2gray(img);
        end
        img_texton = MRS4fast(imfilter(img_f, fspecial('gaussian', 3, 1))) ; 
    otherwise
end

% segment type
TspImg = spImg(:) ;
num_SP = max(TspImg) ;

% compute the mean of color descriptors of each segment
if clrMode>0,
    %{
    descSPClr = zeros(num_SP, isz_ch) ;
    for i=1:num_SP,
        tmp_ind = find(TspImg==i) ;
        descSPClr(i,:) = mean(img_clr(tmp_ind,:),1) ;
    end
    %}
    % The above routine is implemented in mex-c. 
    descSPClr = mex_get_desc_mean_segm(TspImg, img_clr, num_SP) ;
end

% compute the mean of texture descriptors of each segment
if textMode==1,
    %{
    descSPTex = zeros(num_SP, size(img_texton,2)) ;
    for i=1:num_SP,
        tmp_ind = find(TspImg==i) ;
        descSPTex(i,:) = mean(img_clr(tmp_ind,:),1) ;
    end
    %}
    % The above routine is implemented in mex-c. 
    descSPTex = mex_get_desc_mean_segm(TspImg, img_texton, num_SP) ;
end

end
