function [EvalSet aggloClID] = gen_eval_list( descSpCLR, locNet, ...
                spImg, AdjMat, NUM_EVAL_PNT, CLUSTER_THRES)
%
%	Generate Evaluation points per image
%   Source can be placed in only evaluation points.
%
% -------
% INPUTS:
%
%   descSpCLR           : Color descriptors of SPs ([M x d])
%                         where M: # ofSPs, d: feat dimension (ex. rgb: 3)
%   locNet              : intra-network of an image ([M x M])
%   spImg               : Superpixel image
%   AdjMat              : Neighborhood connectivity of SPs in an
%                         image ([M x M])
%   NUM_EVAL_PNT        : number of evaluation points
%   CLUSTER_THRES       : In order to be a evaluation point, the
%                         cluster size should be larger than this value.
%
% --------
% OUTPUTS:
%
%   EvalSet             : The SPs where submodularity will be
%                         checked (local SP indices). ([NUM_EVAL_PNT x 1])
%   aggloClID           : Agglomerative clustering result ([Mx1])
%
% -------------
%
% Gunhee Kim
%

% % --------------------------------------------------------------%
%     Parameters
% % --------------------------------------------------------------%

EPS = 10^-6 ;
BIGV = 10^6 ;
SIM_THRES = 0.01 ;

%CLUSTER_THRES = 3 ;

nSP = size(AdjMat,1) ;

% % --------------------------------------------------------------%
% Run Agglomerative clustering to get the evaluation set.
% % --------------------------------------------------------------%

% Neighboring relations between SPs.
AdjMat = AdjMat+AdjMat' ;

% pairwise distance between SPs in feature space
% if a pair of SPs is not connected, then pairwise distance is 0.
distSP = fn_dist_l2(descSpCLR') ;
distSP(distSP<=0) = EPS ;
distSP(AdjMat==0) = 0 ;
distSP = sparse(distSP) ;

% make distSP to be available on Matlab's linkage function
distSPNgh = BIGV*ones(nSP) ;
distSPNgh(AdjMat>0) = distSP(AdjMat>0) ;
distSPNgh = distSPNgh(tril(ones(nSP,nSP),-1)>0) ;

% agglomerative clustering
Z = linkage(distSPNgh', 'single');
aggloClID = cluster(Z, 'maxclust', NUM_EVAL_PNT);

% if some cluster has a single element, merge it with the
% closest cluster in spatial space.
while 1,
    % get the size of each cluster.
    cntClID = histc(aggloClID,1:NUM_EVAL_PNT) ;
    % find the insiginificant clusters
    insigClID = find(cntClID<CLUSTER_THRES & cntClID>0) ;
    % if there's no insiginificant cluster, break the loop.
    if isempty(insigClID), break; end

    % for each insignificant cluster, merge it with closest neighbor
    while 1,
        % if it's done, break the loop.
        if isempty(insigClID), break; end

        % get the SP ID of the insignificant cluster whose size is less than CLUSTER_THRES
        loc_SPID = find(aggloClID==insigClID(1)) ;

        loc_min_v = BIGV ; loc_min_SPID = 0 ;
        for j=1:length(loc_SPID),
            % get the feature distances of neighbors
            loc_ngh_dist = distSP(loc_SPID(j),:) ;
            % only consider nonzero element
            loc_ngh_ind = find(loc_ngh_dist>0) ;
            % prevent the cloest neighbor to be the one in the same cluster
            [~, ia ib] = setxor(loc_ngh_ind, loc_SPID) ;
            loc_ngh_ind = loc_ngh_ind(ia) ;
            % find the closest SP
            [loc_minv loc_mini] = min(loc_ngh_dist(loc_ngh_ind)) ;
            % if it's the minimum, save it.
            if loc_minv < loc_min_v,
                loc_min_v = loc_minv ;
                loc_min_SPID = loc_ngh_ind(loc_mini) ;
            end
        end

        % get the clusterID of the nearest SP
        min_ngh_aggloID = aggloClID(loc_min_SPID) ;
        aggloClID(loc_SPID) = min_ngh_aggloID ;
        % by merging, the new cluster is not insignificant
        % cluster any more, remove it.
        if length(find(aggloClID==min_ngh_aggloID))>=CLUSTER_THRES,
            insigClID(insigClID==min_ngh_aggloID) = [];
        end
        % remove the merged cluster out of our consideration.
        insigClID(1) = [] ;
    end
end

% significant clusters
cntClID = histc(aggloClID,1:NUM_EVAL_PNT) ;
sigClID = find(cntClID>=CLUSTER_THRES) ;
NUM_EVAL_PNT = length(sigClID) ;

% Get evaluation points.
FaggloClID = aggloClID ;

% first get the SPs along the image boundaries.
bdSP = unique([spImg(1,:) spImg(end,:) spImg(:,1)' spImg(:,end)']) ;
bdSP(bdSP==0) = [] ;

% ignore SP near boundaries
bdLink = zeros(nSP,1) ; bdLink(bdSP) = 1 ;
tmpMat = fn_normout([[locNet 2*bdLink] ; [ones(1,nSP) 0]]) ;
for i=1:NUM_EVAL_PNT,
    cl_ind = find(aggloClID==sigClID(i)) ;
    FaggloClID(cl_ind) = i ;
    % the most connected one in each cluster becomes the
    % evaluation point of the cluster.
    %{
    pval = sum(fn_normout(locNet(cl_ind,cl_ind))) ;
    [max_val max_ind] = max(pval) ;
    EvalSet(i) = cl_ind(max_ind) ;
    %}
    ncl_ind = length(cl_ind) ;
    % expected number of visits before absorption.
    % absorption occurs at cluster/image boundaries
    tmpval = ones(1, ncl_ind) / (eye(ncl_ind) - tmpMat(cl_ind,cl_ind)) ;
    [max_val max_ind] = max(tmpval) ;
    EvalSet(i) = cl_ind(max_ind) ;
end

% ignore trivial positions further

% run clustering from EvalSet
%[cluster_id] = run_cluster_from_rankset(locNet, EvalSet, 1:NUM_EVAL_PNT) ;
%sigClID = find(histc(cluster_id, 1:NUM_EVAL_PNT)>=CLUSTER_THRES) ;
%EvalSet = EvalSet(sigClID) ;

% sort evaluation points
[EvalSet sort_ind] = sort(EvalSet) ;

% change aggloClID accordingly
for i=1:NUM_EVAL_PNT,
    aggloClID(FaggloClID==sort_ind(i)) = i ;
end

end

% normalize to [0,1]
function data = normalize(data)

    sr = size(data,1) ;
    sc = size(data,2) ;

    % normalize to [0,1]
    minDist = min(data(:)) ;
    maxDist = max(data(:)) ;
    data  = data - minDist ;
    diff = maxDist - minDist ;

    if diff==0,
        weights = ones(sr, sc) ;
    else
        data = data / diff ;
    end

end
