function [spImg, AdjMat, cntSeg] = run_superpixel(img, NUM_SP, TYPE_SUPERPIXEL, bDRAW)
%
%	Generate Superpixels
%
% -------
% INPUTS:
%
%   flimg           : image path to be superpixeled
%	NUM_SP          : number of superpixels
%   pos_pole        : nodes attached to sources
%   TYPE_SUPERPIXEL : segmentation method
%   bDRAW           : if 1, draw the result.
%
% --------
% OUTPUTS:
%
%   spImg           : segmet images,
%   AdjMat          : Neighborhood matrix btw segments
%   cntSeg          : center point of each segment.
%
% -------------
%
% Gunhee Kim

% default parameters
if nargin<2, NUM_SP = 500 ;             end
if nargin<3, TYPE_SUPERPIXEL = 3 ;      end
if nargin<4, bDRAW = 0 ;                end



%%%%%%%%% Get Super-pixels  %%%%%%%%%%%%%%%%%%
switch TYPE_SUPERPIXEL,
% Method 1: Get Watershed pixels based on the edge information
% fast but poor quality.
case 1,
    I_gray = rgb2gray(img);
    hy = fspecial('sobel');
    hx = hy';
    Iy = imfilter(double(I_gray), hy, 'replicate');
    Ix = imfilter(double(I_gray), hx, 'replicate');
    gradmag = sqrt(Ix.^2 + Iy.^2);
    spImg = watershed(gradmag);

    %I2 = imcomplement(gradmag);
    %20 is the height threshold for suppressing %shallow minima
    %I3 = imhmin(I2,5);
    %spImg = watershed(I2);

    % number of superpixels
    nSeg = max(max(spImg)) ;
    % get the binary boundary image (1: boundaries)
    bdImg = ~spImg ;
    % get adjacency matrix
    AdjMat = sparse(mex_adjacency_graph_zero(spImg, nSeg)) ;

% Method 2: Super-pixel code by Greg Mori
% http://www.cs.sfu.ca/~mori/research/superpixels
% good quality but extremely slow.
case 2,
    spImg = get_sp_ncuts(im2double(img), NUM_SP);
    % save tmp_ncuts.mat spImg
    % load tmp_ncuts.mat

    % number of segments
    nSeg = max(max(spImg)) ;
    % get the binary boundary image (1: boundaries)
    [cx,cy] = gradient(spImg);
    bdImg = (abs(cx)+abs(cy))~=0;
    bdImg = bwmorph(bdImg,'skel',Inf) ;
    clear cx cy

    % get adjacency matrix
    AdjMat = sparse(mex_adjacency_graph(spImg, nSeg)) ;

% Method 3: Super-pixel code by Turbopixel
% http://www.cs.toronto.edu/~babalex/research.html
case 3,
    % get the segment image (0: boundaries, nonzeros: segments)
    spImg = get_sp_turbo(fn_im2double(img), NUM_SP);

    % number of segments
    nSeg = max(spImg(:)) ;
    % get the binary boundary image (1: boundaries)
    bdImg = ~spImg ;
    % get adjacency matrix
    AdjMat = sparse(mex_adjacency_graph_zero(spImg, nSeg)) ;

% Method 4: SLIC of VLFeat.org
% http://www.vlfeat.org/overview/slic.html
otherwise,
    % get the segment image (0: boundaries, nonzeros: segments)
    spImg = get_sp_slic(fn_im2double(img), NUM_SP);
    spImg = double(spImg) ;

    % number of segments
    nSeg = max(max(spImg)) ;
    % get the binary boundary image (1: boundaries)
    % bdImg = ~spImg ;

    % get adjacency matrix
    AdjMat = get_adjacency_graph(spImg) ;
end

% center coordinates of each segment
cntSeg = zeros(nSeg,2) ;
for i=1:nSeg,
    [r c] = find(spImg==i) ;
    cntSeg(i,:) = round([mean(r) mean(c)]) ;
end
%spImg = uint16(spImg) ;
%cntSeg = uint16(cntSeg) ;

% draw results?
if bDRAW>0
    % add library path
    if ~exist('mex_draw_thick_lines_on_img'),
        addpath('lib/draw_thick_lines') ;
    end

    % show the super-pixel results.
    figure;
    subplot(1,2,1); imshow(img);
    subplot(1,2,2); imshow(label2rgb(spImg, 'jet', 'w', 'shuffle'));

    % boundary image
    [FX FY] = gradient(double(spImg)) ;
    bdImg = abs(FX) + abs(FY) ;
    bdImg(bdImg~=0) = 1 ;
    clear FX FY ;

    % draw graph
    [adj_r adj_c] = find(AdjMat) ;
    num_nghpair = length(adj_r) ;
    contour_color = [255,0,0];
    disp_img = img ;
    [size_r size_c size_ch] = size(disp_img) ;
    disp_img = reshape(disp_img, [size_r*size_c size_ch]) ;
    for cc=1:3, disp_img(bdImg==1,cc) = contour_color(cc) ; end
    disp_img = reshape(disp_img, [size_r size_c size_ch]) ;

    %disp_img = mex_draw_thick_lines_on_img(disp_img, [cntSeg(adj_r,1) cntSeg(adj_c,1) cntSeg(adj_r,2) cntSeg(adj_c,2)], ones(num_nghpair), ...
    %25*ones(num_nghpair,3)) ;
    figure; imagesc(disp_img); axis equal; axis off;
    label_img = label2rgb(spImg, 'jet', 'w', 'shuffle') ;
    label_img = mex_draw_thick_lines_on_img(label_img, [cntSeg(adj_r,1) cntSeg(adj_c,1) cntSeg(adj_r,2) cntSeg(adj_c,2)], ones(num_nghpair), ...
        25*ones(num_nghpair,3)) ;
    %figure; imshow(label_img); axis equal; axis off;
    keyboard;
end
end
